@@ -1,7 +1,7 @@
 /*
  * radius.h	Constants of the radius protocol.
  *
- * Version:	$Id: radius.h,v 1.23 2001/07/26 19:10:19 aland Exp $
+ * Version:	$Id: radius.h,v 1.31 2001/11/11 05:17:10 fcusack Exp $
  *
  */
 
@@ -47,7 +47,12 @@
 #define PW_REPLY_MESSAGE		18
 #define PW_CALLBACK_NUMBER		19
 #define PW_CALLBACK_ID			20
+#if 0
+/*
+ *  Deprecated, and no longer used.
+ */
 #define PW_EXPIRATION			21
+#endif
 #define PW_FRAMED_ROUTE			22
 #define PW_FRAMED_IPXNET		23
 #define PW_STATE			24
@@ -69,16 +74,30 @@
 #define PW_ACCT_SESSION_TIME		46
 #define PW_ACCT_INPUT_PACKETS		47
 #define PW_ACCT_OUTPUT_PACKETS		48
-#define PW_ACCT_TERMINATE_CAUSE	49
+#define PW_ACCT_TERMINATE_CAUSE		49
+
+#define PW_EVENT_TIMESTAMP		55
 
 #define PW_CHAP_CHALLENGE		60
 #define PW_NAS_PORT_TYPE		61
 #define PW_PORT_LIMIT			62
-#define PW_CONNECT_INFO			77
 
+#define PW_ARAP_PASSWORD		70
+#define PW_ARAP_FEATURES		71
+#define PW_ARAP_ZONE_ACCESS		72
+#define PW_ARAP_SECURITY		73
+#define PW_ARAP_SECURITY_DATA		74
+#define PW_PASSWORD_RETRY		75
+#define PW_PROMPT			76
+#define PW_CONNECT_INFO			77
+#define PW_CONFIGURATION_TOKEN		78
 #define PW_EAP_MESSAGE                  79
 #define PW_MESSAGE_AUTHENTICATOR        80
 
+#define PW_ARAP_CHALLENGE_RESPONSE	84
+#define PW_NAS_PORT_ID_STRING  		87
+#define PW_FRAMED_POOL			89
+
 #define PW_FALL_THROUGH			500
 #define PW_ADD_PORT_TO_IP_ADDRESS	501
 #define PW_EXEC_PROGRAM			502
@@ -92,6 +111,7 @@
 #define PW_CONNECT_RATE			1007
 #define PW_ADD_PREFIX			1008
 #define PW_ADD_SUFFIX			1009
+#define PW_EXPIRATION			1010
 #define PW_USER_CATEGORY		1029
 #define PW_GROUP_NAME			1030
 #define PW_HUNTGROUP_NAME		1031
@@ -113,6 +133,8 @@
 #define LDAP_USERDN			1053
 #define PW_NS_MTA_MD5_PASSWORD		1054
 #define PW_SQL_USER_NAME  		1055
+#define PW_MODULE_MESSAGE		1056
+#define PW_X99_FAST			1057
 
 /*
  *	Integer Translations
@@ -160,7 +182,8 @@
 #define PW_AUTHTYPE_SECURID		2
 #define PW_AUTHTYPE_CRYPT		3
 #define PW_AUTHTYPE_REJECT		4
-#define PW_AUTHTYPE_EAP                 252
+#define PW_AUTHTYPE_ACTIVCARD		5
+#define PW_AUTHTYPE_EAP                 6
 #define PW_AUTHTYPE_PAM			253
 #define PW_AUTHTYPE_ACCEPT		254
 
