@@ -4,7 +4,7 @@
  * radiusd.h	Structures, prototypes and global variables
  *		for the FreeRADIUS server.
  *
- * Version:	$Id: radiusd.h,v 1.91 2001/09/19 14:20:27 aland Exp $
+ * Version:	$Id: radiusd.h,v 1.95 2001/11/27 22:44:24 kkalev Exp $
  *
  */
 #include "libradius.h"
@@ -93,6 +93,7 @@ typedef struct realm {
 	char			realm[64];
 	char			server[64];
 	uint32_t		ipaddr;
+	uint32_t		acct_ipaddr;
 	u_char			secret[32];
 	int			auth_port;
 	int			acct_port;
@@ -156,7 +157,14 @@ typedef struct main_config_t {
 #endif
 
 /* for paircompare_register */
-typedef int (*RAD_COMPARE_FUNC)(void *instance, VALUE_PAIR *, VALUE_PAIR *, VALUE_PAIR *, VALUE_PAIR **);
+typedef int (*RAD_COMPARE_FUNC)(void *instance, REQUEST *,VALUE_PAIR *, VALUE_PAIR *, VALUE_PAIR *, VALUE_PAIR **);
+
+typedef enum radlog_dest_t {
+  RADLOG_FILES = 0,
+  RADLOG_SYSLOG,
+  RADLOG_STDOUT,
+  RADLOG_STDERR
+} radlog_dest_t;
 
 /*
  *	Global variables.
@@ -169,6 +177,7 @@ extern const char	*radlog_dir;
 extern const char	*radlib_dir;
 extern const char	*radius_dir;
 extern const char	*radius_libdir;
+extern radlog_dest_t	radlog_dest;
 extern uint32_t		expiration_seconds;
 extern int		use_dbm;
 extern int		log_stripped_names;
@@ -278,9 +287,9 @@ int		paircompare_register(int attr, int otherattr,
 				     RAD_COMPARE_FUNC func,
 				     void *instance);
 void		paircompare_unregister(int attr, RAD_COMPARE_FUNC func);
-int		paircmp(VALUE_PAIR *request, VALUE_PAIR *check,
+int		paircmp(REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check,
 			VALUE_PAIR **reply);
-int             simplepaircmp(VALUE_PAIR *, VALUE_PAIR *);
+int		simplepaircmp(REQUEST *, VALUE_PAIR *, VALUE_PAIR *);
 void		pair_builtincompare_init(void);
 
 /* xlat.c */
@@ -288,6 +297,10 @@ typedef int (*RADIUS_ESCAPE_STRING)(char *out, int outlen, const char *in);
 
 int            radius_xlat(char * out, int outlen, const char *fmt,
 			   REQUEST * request, RADIUS_ESCAPE_STRING func);
+typedef int (*RAD_XLAT_FUNC)(void *instance, REQUEST *, char *, char *, int, RADIUS_ESCAPE_STRING func);
+int		xlat_register(char *module, RAD_XLAT_FUNC func, void *instance);
+void		xlat_unregister(char *module, RAD_XLAT_FUNC func);
+
 
 #ifdef WITH_THREAD_POOL
 /* threads.c */
