@@ -1,11 +1,11 @@
 /*
  * dict.c	Routines to read the dictionary file.
  *
- * Version:	$Id: dict.c,v 1.25 2001/03/29 18:08:18 aland Exp $
+ * Version:	$Id: dict.c,v 1.28 2001/10/31 17:01:13 cparker Exp $
  *
  */
 
-static const char rcsid[] = "$Id: dict.c,v 1.25 2001/03/29 18:08:18 aland Exp $";
+static const char rcsid[] = "$Id: dict.c,v 1.28 2001/10/31 17:01:13 cparker Exp $";
 
 #include	"autoconf.h"
 
@@ -24,7 +24,6 @@ static DICT_ATTR	*dictionary_attributes = NULL;
 static DICT_VALUE	*dictionary_values = NULL;
 static DICT_VENDOR	*dictionary_vendors = NULL;
 
-static int		vendorno = 1;
 static const char *dtypes[] = {
 	"string",
 	"integer",
@@ -76,7 +75,6 @@ static void dict_free(void)
 	dictionary_attributes = NULL;
 	dictionary_values = NULL;
 	dictionary_vendors = NULL;
-	vendorno = 1;
 
 	memset(base_attributes, 0, sizeof(base_attributes));
 }
@@ -88,6 +86,11 @@ int dict_addvendor(const char *name, int value)
 {
 	DICT_VENDOR *vval;
 
+	if (value >= (1 << 16)) {
+	       	librad_log("dict_addvendor: Cannot handle vendor ID larger than 65535");
+		return -1;
+	}
+
 	if (strlen(name) > (sizeof(vval->vendorname) -1)) {
 		librad_log("dict_addvendor: vendor name too long");
 		return -1;
@@ -99,7 +102,6 @@ int dict_addvendor(const char *name, int value)
 	}
 	strcpy(vval->vendorname, name);
 	vval->vendorpec  = value;
-	vval->vendorcode = vendorno++;
 
 	/* Insert at front. */
 	vval->next = dictionary_vendors;
@@ -108,7 +110,7 @@ int dict_addvendor(const char *name, int value)
 	return 0;
 }
 
-int dict_addattr(const char *name, int vendor, int type, int value)
+int dict_addattr(const char *name, int vendor, int type, int value, ATTR_FLAGS flags)
 {
 	static int      max_attr = 0;
 	DICT_ATTR	*attr;
@@ -152,6 +154,8 @@ int dict_addattr(const char *name, int vendor, int type, int value)
 	strcpy(attr->name, name);
 	attr->attr = value;
 	attr->type = type;
+	attr->flags = flags;
+
 	if (vendor) {
 		attr->attr |= (vendor << 16);
 	} else if ((attr->attr >= 0) && (attr->attr < 256)) {
@@ -216,8 +220,8 @@ static int my_dict_init(const char *dir, const char *fn, const char *src_file, i
 	char	valstr[256];
 	char	attrstr[256];
 	char	typestr[256];
-	char	vendorstr[256];
-	char	*p;
+	char	optstr[256];
+	char	*p, *s, *c;
 	char	*keyword;
 	char	*data;
 	int	line = 0;
@@ -228,6 +232,7 @@ static int my_dict_init(const char *dir, const char *fn, const char *src_file, i
 	int	is_attrib;
 	int	vendor_usr_seen = 0;
 	int	is_nmc;
+	ATTR_FLAGS  flags;
 
 	if (strlen(fn) >= sizeof(dirtmp) / 2 ||
 	    strlen(dir) >= sizeof(dirtmp) / 2) {
@@ -310,9 +315,9 @@ static int my_dict_init(const char *dir, const char *fn, const char *src_file, i
 		if (is_attrib) {
 
 			vendor = 0;
-			vendorstr[0] = 0;
+			optstr[0] = 0;
 			if(sscanf(data, "%s%s%s%s", namestr,
-					valstr, typestr, vendorstr) < 3) {
+					valstr, typestr, optstr) < 3) {
 				librad_log(
 					"dict_init: %s[%d]: invalid ATTRIBUTE line",
 					fn, line);
@@ -324,7 +329,7 @@ static int my_dict_init(const char *dir, const char *fn, const char *src_file, i
 			 *	We might need to add USR to the list of
 			 *	vendors first.
 			 */
-			if (is_nmc && vendorstr[0] == 0) {
+			if (is_nmc && optstr[0] == 0) {
 				if (!vendor_usr_seen) {
 					if (dict_addvendor("USR", VENDORPEC_USR) < 0) {
 						librad_log("dict_init: %s[%d]: %s", fn, line, librad_errstr);
@@ -332,7 +337,7 @@ static int my_dict_init(const char *dir, const char *fn, const char *src_file, i
 					}
 					vendor_usr_seen = 1;
 				}
-				strcpy(vendorstr, "USR");
+				strcpy(optstr, "USR");
 			}
 
 			/*
@@ -364,35 +369,80 @@ static int my_dict_init(const char *dir, const char *fn, const char *src_file, i
 			/*
 			 *	Ignore comments
 			 */
-			if (vendorstr[0] == '#') {
-				vendorstr[0] = '\0';
+			if (optstr[0] == '#') {
+				optstr[0] = '\0';
 			}
 
 			/*
 			 *	Only look up the vendor if the string
 			 *	is non-empty.
 			 */
-			if (vendorstr[0]) {
-				vendor = dict_vendorname(vendorstr);
-				if (!vendor) {
-					librad_log(
-						"dict_init: %s[%d]: unknown vendor %s",
-						fn, line, vendorstr);
-					return -1;
+
+			memset(&flags, 0, sizeof(flags));
+			s = strtok(optstr, ",");
+			while(s) {
+			        if (strcmp(s, "has_tag") == 0 ||
+				    strcmp(s, "has_tag=1") == 0) {
+				         /* Boolean flag, means this is a
+					    tagged attribute */
+				         flags.has_tag = 1;
 				}
+				else if (strncmp(s, "len+=", 5) == 0 ||
+					 strncmp(s, "len-=", 5) == 0) { 
+				          /* Length difference, to accomodate
+					     braindead NASes & their vendors */
+				          flags.len_disp = strtol(s + 5, &c, 0);
+					  if (*c) {
+					        librad_log(
+							   "dict_init: %s[%d] invalid option %s",
+							   fn, line, s);
+						return -1;
+					  }
+					  if (s[3] == '-') {
+					        flags.len_disp = -flags.len_disp;
+					  }
+				}
+				else if (strncmp(s, "encrypt=", 8) == 0) {
+				          /* Encryption method, defaults to 0 (none).
+					     Currently valid is just type 1,
+					     Tunnel-Password style, which can only
+					     be applied to strings. */    
+				          flags.encrypt = strtol(s + 8, &c, 0);
+					  if (*c) {
+					        librad_log(
+							   "dict_init: %s[%d] invalid option %s",
+							   fn, line, s);
+						return -1;
+					  }
+				}
+				else {
+				          /* Must be a vendor 'flag'... */
+				          if (strncmp(s, "vendor=", 5) == 0) {
+					        /* New format */
+					        s += 5;   
+					  }
+                         
+					  vendor = dict_vendorname(s);
+					  if (!vendor) {
+					        librad_log(
+							   "dict_init: %s[%d]: unknown vendor %s",
+							   fn, line, optstr);
+						return -1;
+					  }
+					  if (block_vendor && optstr[0] &&
+					      (block_vendor != vendor)) {
+					        librad_log(
+							   "dict_init: %s[%d]: mismatched vendor %s within BEGIN-VENDOR/END-VENDOR block",
+							   fn, line, optstr);
+						return -1;
+					  }
+				}
+				s = strtok(NULL, ",");
 			}
-
-			if (block_vendor && vendorstr[0] &&
-			    (block_vendor != vendor)) {
-				librad_log(
-					"dict_init: %s[%d]: mismatched vendor %s within BEGIN-VENDOR/END-VENDOR block",
-					fn, line, vendorstr);
-				return -1;
-			}
-
+ 
 			if (block_vendor) vendor = block_vendor;
 
-			if (dict_addattr(namestr, vendor, type, value) < 0) {
+			if (dict_addattr(namestr, vendor, type, value, flags) < 0) {
 				librad_log("dict_init: %s[%d]: %s",
 					   fn, line, librad_errstr);
 				return -1;
@@ -486,19 +536,19 @@ static int my_dict_init(const char *dir, const char *fn, const char *src_file, i
 		}
 
 		if (DICT_STRCMP(keyword, "BEGIN-VENDOR") == 0) {
-			vendorstr[0] = 0;
-			if (sscanf(data, "%s", vendorstr) != 1) {
+			optstr[0] = 0;
+			if (sscanf(data, "%s", optstr) != 1) {
 				librad_log(
 				"dict_init: %s[%d] invalid BEGIN-VENDOR entry",
 					fn, line);
 				return -1;
 			}
 
-			vendor = dict_vendorname(vendorstr);
+			vendor = dict_vendorname(optstr);
 			if (!vendor) {
 				librad_log(
 					"dict_init: %s[%d]: unknown vendor %s",
-					fn, line, vendorstr);
+					fn, line, optstr);
 				return -1;
 			}
 			block_vendor = vendor;
@@ -506,26 +556,26 @@ static int my_dict_init(const char *dir, const char *fn, const char *src_file, i
 		}
 
 		if (DICT_STRCMP(keyword, "END-VENDOR") == 0) {
-			vendorstr[0] = 0;
-			if (sscanf(data, "%s", vendorstr) != 1) {
+			optstr[0] = 0;
+			if (sscanf(data, "%s", optstr) != 1) {
 				librad_log(
 				"dict_init: %s[%d] invalid END-VENDOR entry",
 					fn, line);
 				return -1;
 			}
 
-			vendor = dict_vendorname(vendorstr);
+			vendor = dict_vendorname(optstr);
 			if (!vendor) {
 				librad_log(
 					"dict_init: %s[%d]: unknown vendor %s",
-					fn, line, vendorstr);
+					fn, line, optstr);
 				return -1;
 			}
 
 			if (vendor != block_vendor) {
 				librad_log(
 					"dict_init: %s[%d]: END-VENDOR %s does not match any previous BEGIN-VENDOR",
-					fn, line, vendorstr);
+					fn, line, optstr);
 				return -1;
 			}
 			block_vendor = 0;
@@ -640,35 +690,7 @@ DICT_VALUE * dict_valbyname(int attr, const char *name)
 }
 
 /*
- *	Get the PEC (Private Enterprise Code) of the vendor
- *	based on its internal number.
- */
-int dict_vendorpec(int code)
-{
-	DICT_VENDOR	*v;
-
-	for (v = dictionary_vendors; v; v = v->next)
-		if (v->vendorcode == code)
-			break;
-
-	return v ? v->vendorpec : 0;
-}
-
-/*
- *	Get the internal code of the vendor based on its PEC.
- */
-int dict_vendorcode(int pec)
-{
-	DICT_VENDOR	*v;
-
-	for (v = dictionary_vendors; v; v = v->next)
-		if (v->vendorpec == pec)
-			break;
-	return v ? v->vendorcode : 0;
-}
-
-/*
- *	Get the vendor code based on the vendor name
+ *	Get the vendor PEC based on the vendor name
  */
 int dict_vendorname(const char *name)
 {
@@ -679,7 +701,7 @@ int dict_vendorname(const char *name)
 	 */
 	for (v = dictionary_vendors; v; v = v->next) {
 		if (DICT_STRCMP(name, v->vendorname) == 0) {
-			return v->vendorcode;
+			return v->vendorpec;
 		}
 	}
 
