@@ -1,11 +1,11 @@
 /*
  * print.c	Routines to print stuff.
  *
- * Version:	$Id: print.c,v 1.14 2001/03/12 16:48:14 aland Exp $
+ * Version:	$Id: print.c,v 1.18 2001/11/08 01:04:33 cparker Exp $
  *
  */
 
-static const char rcsid[] = "$Id: print.c,v 1.14 2001/03/12 16:48:14 aland Exp $";
+static const char rcsid[] = "$Id: print.c,v 1.18 2001/11/08 01:04:33 cparker Exp $";
 
 #include	"autoconf.h"
 
@@ -97,26 +97,45 @@ int vp_prints_value(char * out, int outlen, VALUE_PAIR *vp, int delimitst)
 			if (vp->attribute == PW_NAS_PORT_ID)
 				a = (char *)vp->strvalue;
 			else {
-				if (delimitst) {
-				  buf[0] = '"';
-				  librad_safeprint((char *)vp->strvalue,
-					  vp->length, buf + 1, sizeof(buf) - 2);
-				  strcat(buf, "\"");
+				if (delimitst && vp->flags.has_tag) {
+				        /* Tagged attribute: print delimter and ignore tag */
+				        buf[0] = '"';
+					librad_safeprint((char *)(vp->strvalue),
+							 vp->length, buf + 1, sizeof(buf) - 2);
+					strcat(buf, "\"");
+				} else if (delimitst) {
+				        /* Non-tagged attribute: print delimter */
+				        buf[0] = '"';
+					librad_safeprint((char *)vp->strvalue,
+							 vp->length, buf + 1, sizeof(buf) - 2);
+					strcat(buf, "\"");
 				} else {
-				  librad_safeprint((char *)vp->strvalue,
-					  vp->length, buf, sizeof(buf));
+				        /* Non-tagged attribute: no delimiter */
+				        librad_safeprint((char *)vp->strvalue,
+							 vp->length, buf, sizeof(buf));
 				}
-
 				a = buf;
 			}
 			break;
 		case PW_TYPE_INTEGER:
-			if ((v = dict_valbyattr(vp->attribute, vp->lvalue))
-				!= NULL)
-				a = v->name;
-			else {
-				snprintf(buf, sizeof(buf), "%d", vp->lvalue);
-				a = buf;
+		        if ( vp->flags.has_tag ) {
+			        /* Attribute value has a tag, need to ignore it */
+			        if ((v = dict_valbyattr(vp->attribute, (vp->lvalue & 0xffffff)))
+				    != NULL)
+				        a = v->name;
+				else {
+				        snprintf(buf, sizeof(buf), "%u", (vp->lvalue & 0xffffff));
+				        a = buf;
+				}
+			} else {
+			        /* Normal, non-tagged attribute */
+			        if ((v = dict_valbyattr(vp->attribute, vp->lvalue))
+				    != NULL)
+				        a = v->name;
+				else {
+				        snprintf(buf, sizeof(buf), "%u", vp->lvalue);
+					a = buf;
+				}
 			}
 			break;
 		case PW_TYPE_DATE:
@@ -178,9 +197,20 @@ int vp_prints(char *out, int outlen, VALUE_PAIR *vp)
 		return 0;
 	}
 
-	snprintf(out, outlen, "%s = ", vp->name);
-	len = strlen(out);
-	vp_prints_value(out + len, outlen - len, vp, 1);
+	if( vp->flags.has_tag ) {
+
+		snprintf(out, outlen, "%s:%d = ", vp->name, vp->flags.tag);
+		
+		len = strlen(out);
+		vp_prints_value(out + len, outlen - len, vp, 1);
+
+	} else {
+
+	        snprintf(out, outlen, "%s = ", vp->name);
+		len = strlen(out);
+		vp_prints_value(out + len, outlen - len, vp, 1);
+
+	}	  
 
 	return strlen(out);
 }
