@@ -1,16 +1,21 @@
 /*
  * radius.c	Functions to send/receive radius packets.
  *
- * Version:	$Id: radius.c,v 1.55 2001/09/05 18:51:24 aland Exp $
+ * Version:	$Id: radius.c,v 1.63 2001/11/29 09:45:00 3APA3A Exp $
  *
  */
 
-static const char rcsid[] = "$Id: radius.c,v 1.55 2001/09/05 18:51:24 aland Exp $";
+static const char rcsid[] = "$Id: radius.c,v 1.63 2001/11/29 09:45:00 3APA3A Exp $";
 
 #include	"autoconf.h"
+#include	"md5.h"
 
 #include	<stdlib.h>
+
+#if HAVE_UNISTD_H
 #include	<unistd.h>
+#endif
+
 #include	<fcntl.h>
 #include	<string.h>
 #include	<ctype.h>
@@ -42,6 +47,19 @@ static const char rcsid[] = "$Id: radius.c,v 1.55 2001/09/05 18:51:24 aland Exp
  */
 #define PACKET_DATA_LEN 1600
 
+/*
+ *	The maximum number of attributes which we allow in an incoming
+ *	request.  If there are more attributes than this, the request
+ *	is rejected.
+ *
+ *	This helps to minimize the potential for a DoS, when an
+ *	attacker spoofs Access-Request packets, which don't have a
+ *	Message-Authenticator attribute.  This means that the packet
+ *	is unsigned, and the attacker can use resources on the server,
+ *	even if the end request is rejected.
+ */
+int librad_max_attributes = 0;
+
 typedef struct radius_packet_t {
   uint8_t	code;
   uint8_t	id;
@@ -105,15 +123,15 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 		  int			vendorcode, vendorpec;
 		  u_short		total_length;
 		  int			len, allowed;
-		  uint8_t		*msg_auth_ptr = NULL;
-		  
-		  hdr = (radius_packet_t *) malloc(PACKET_DATA_LEN);
-		  if (!hdr) {
-		    librad_log("Out of memory");
-		    return -1;
-		  }
-		  packet->data = (uint8_t *) hdr;
+		  int			msg_auth_offset = 0;
+		  uint8_t		data[4096];
 		  
+		  /*
+		   *	Use memory on the stack, until we know how
+		   *	large the packet will be.
+		   */
+		  hdr = (radius_packet_t *) data;
+
 		  /*
 		   *	Build standard header
 		   */
@@ -158,7 +176,7 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 				   */
 				  reply->length = AUTH_VECTOR_LEN;
 				  memset(reply->strvalue, 0, AUTH_VECTOR_LEN);
-				  msg_auth_ptr = ptr;
+				  msg_auth_offset = total_length;
 			  }
 
 			  /*
@@ -274,8 +292,41 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 					  memcpy(reply->strvalue, digest, AUTH_VECTOR_LEN );
 					  reply->length = AUTH_VECTOR_LEN;
 				  }
+
+				  /* 
+				   *    Encrypt Tunnel-Password
+				   *    here -- cparker@starnetusa.net
+				   */
+
+				  /*
+				   *	Encrypt with Tunnel-Password style if needed.
+				   */
+				  if (reply->flags.encrypt == 2) {
+
+				    rad_tunnel_pwencode(reply->strvalue,
+							&(reply->length),
+							secret,
+							packet->vector);
+				    
+				  }
+			  
+
 				  len = reply->length;
-				  
+
+				  /*
+				   *    Set the TAG at the beginning of the string if tagged.
+				   *    If tag value is not valid for tagged attribute, make 
+				   *    it 0x00 per RFC 2868.  -cparker
+				   */
+				  if(reply->flags.has_tag) {
+				          len++;
+					  if(TAG_VALID(reply->flags.tag)) {
+					          *ptr++ = reply->flags.tag;
+					  } else {
+					          *ptr++ = 0x00;
+					  }
+				  }
+				 
 				  /*
 				   *	Ensure we don't go too far.
 				   *	The 'length' of the attribute
@@ -296,8 +347,14 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 				  
 				  *length_ptr += len;
 				  if (vsa_length_ptr) *vsa_length_ptr += len;
-				  memcpy(ptr, reply->strvalue,len);
-				  ptr += len;
+				  /*
+				   *  If we have tagged attributes we can't assume that
+				   *  len == reply->length.  Use reply->length for copying
+				   *  the string data into the packet.  Use len for the
+				   *  true length of the string+tags.
+				   */
+				  memcpy(ptr, reply->strvalue, reply->length);
+				  ptr += reply->length;
 				  total_length += len;
 				  break;
 				  
@@ -305,10 +362,22 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 			  case PW_TYPE_IPADDR:
 				  *length_ptr += 4;
 				  if (vsa_length_ptr) *vsa_length_ptr += 4;
-				  if (reply->type != PW_TYPE_IPADDR)
-					  lvalue = htonl(reply->lvalue);
-				  else
+
+				  if (reply->type == PW_TYPE_INTEGER ) {
+				          /*  If tagged, the tag becomes the MSB of the value */
+				          if(reply->flags.has_tag) {
+					         /*  Tag must be ( 0x01 -> 0x1F ) OR 0x00  */
+					         if(!TAG_VALID(reply->flags.tag)) {
+						       reply->flags.tag = 0x00;
+						 }
+					         lvalue = htonl((reply->lvalue & 0xffffff) |
+								((reply->flags.tag & 0xff) << 24));
+					  } else {
+					         lvalue = htonl(reply->lvalue);
+					  }
+				  } else {
 					  lvalue = reply->lvalue;
+				  }
 				  memcpy(ptr, &lvalue, 4);
 				  ptr += 4;
 				  total_length += 4;
@@ -325,9 +394,26 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 			   *	attributes.
 			   */
 			  debug_pair(reply);
-		  }
+		  } /* done looping over all attributes */
 
+		  /*
+		   *	Fill in the rest of the fields, and copy
+		   *	the data over from the local stack to
+		   *	the newly allocated memory.
+		   *
+		   *	Yes, all this 'memcpy' is slow, but it means
+		   *	that we only allocate the minimum amount of
+		   *	memory for a request.
+		   */
 		  packet->data_len = total_length;
+		  packet->data = (uint8_t *) malloc(packet->data_len);
+		  if (!packet->data) {
+			  librad_log("Out of memory");
+			  return -1;
+		  }
+		  memcpy(packet->data, data, packet->data_len);
+		  hdr = (radius_packet_t *) packet->data;
+
 		  total_length = htons(total_length);
 		  memcpy(hdr->length, &total_length, sizeof(u_short));
 
@@ -339,12 +425,13 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 		  secretlen = strlen(secret);
 		  if (packet->code != PW_AUTHENTICATION_REQUEST &&
 		      packet->code != PW_STATUS_SERVER) {
+		    MD5_CTX	context;
 		      /*
 		       *	Set the Message-Authenticator attribute,
 		       *	BEFORE setting the reply authentication vector
 		       *	for CHALLENGE, ACCEPT and REJECT.
 		       */
-		      if (msg_auth_ptr) {
+		      if (msg_auth_offset) {
 			      uint8_t calc_auth_vector[AUTH_VECTOR_LEN];
 
 			      switch (packet->code) {
@@ -360,18 +447,22 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 				break;
 			      }
 
-			      memset(msg_auth_ptr + 2, 0, AUTH_VECTOR_LEN);
+			      memset(packet->data + msg_auth_offset + 2, 0,
+				     AUTH_VECTOR_LEN);
 			      lrad_hmac_md5(packet->data, packet->data_len,
 					    secret, secretlen, calc_auth_vector);
-			      memcpy(msg_auth_ptr + 2, calc_auth_vector, AUTH_VECTOR_LEN);
+			      memcpy(packet->data + msg_auth_offset + 2,
+				     calc_auth_vector, AUTH_VECTOR_LEN);
 			      memcpy(hdr->vector, packet->vector, AUTH_VECTOR_LEN);
-		        }
-			memcpy((char *)hdr + packet->data_len, secret, secretlen);
-			librad_md5_calc(digest, (unsigned char *)hdr,
-					packet->data_len + secretlen);
-			memcpy(hdr->vector, digest, AUTH_VECTOR_LEN);
-			memcpy(packet->vector, digest, AUTH_VECTOR_LEN);
-			memset((char *)hdr + packet->data_len, 0, secretlen);
+		      }
+
+		      MD5Init(&context);
+		      MD5Update(&context, packet->data, packet->data_len);
+		      MD5Update(&context, secret, strlen(secret));
+		      MD5Final(digest, &context);
+
+		      memcpy(hdr->vector, digest, AUTH_VECTOR_LEN);
+		      memcpy(packet->vector, digest, AUTH_VECTOR_LEN);
 		  }
 
 		  /*
@@ -379,7 +470,7 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 		   *	AFTER setting the authentication vector
 		   *	only for ACCESS-REQUESTS
 		   */
-		  else if (msg_auth_ptr) {
+		  else if (msg_auth_offset) {
 			  uint8_t calc_auth_vector[AUTH_VECTOR_LEN];
 
 			  switch (packet->code) {
@@ -395,14 +486,15 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 			    break;
 			  }
 
-			  memset(msg_auth_ptr + 2, 0, AUTH_VECTOR_LEN);
+			  memset(packet->data + msg_auth_offset + 2,
+				 0, AUTH_VECTOR_LEN);
 			  lrad_hmac_md5(packet->data, packet->data_len,
 					secret, secretlen, calc_auth_vector);
-			  memcpy(msg_auth_ptr + 2, calc_auth_vector, AUTH_VECTOR_LEN);
+			  memcpy(packet->data + msg_auth_offset + 2,
+				  calc_auth_vector, AUTH_VECTOR_LEN);
 			  memcpy(hdr->vector, packet->vector, AUTH_VECTOR_LEN);
 		  }
 
-
 		  /*
 		   *	If packet->data points to data, then we print out
 		   *	the VP list again only for debugging.
@@ -435,10 +527,10 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
  *	Validates the requesting client NAS.  Calculates the
  *	signature based on the clients private key.
  */
-int calc_acctdigest(RADIUS_PACKET *packet, const char *secret, char *recvbuf, int len)
+static int calc_acctdigest(RADIUS_PACKET *packet, const char *secret)
 {
-	int		secretlen;
 	u_char		digest[AUTH_VECTOR_LEN];
+	MD5_CTX	context;
 
 	/*
 	 *	Older clients have the authentication vector set to
@@ -456,10 +548,15 @@ int calc_acctdigest(RADIUS_PACKET *packet, const char *secret, char *recvbuf, in
 	 *	and calculate the MD5 sum. This must be the same
 	 *	as the original MD5 sum (packet->vector).
 	 */
-	secretlen = strlen(secret);
-	memset(recvbuf + 4, 0, AUTH_VECTOR_LEN);
-	memcpy(recvbuf + len, secret, secretlen);
-	librad_md5_calc(digest, (u_char *)recvbuf, len + secretlen);
+	memset(packet->data + 4, 0, AUTH_VECTOR_LEN);
+	
+	/*
+	 *  MD5(packet + secret);
+	 */
+	MD5Init(&context);
+	MD5Update(&context, packet->data, packet->data_len);
+	MD5Update(&context, secret, strlen(secret));
+	MD5Final(digest, &context);
 
 	/*
 	 *	Return 0 if OK, 2 if not OK.
@@ -474,15 +571,28 @@ int calc_acctdigest(RADIUS_PACKET *packet, const char *secret, char *recvbuf, in
  *	Validates the requesting client NAS.  Calculates the
  *	signature based on the clients private key.
  */
-static int calc_replydigest(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secret, char *recvbuf, int len)
+static int calc_replydigest(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secret)
 {
-	int		secretlen;
 	uint8_t		calc_digest[AUTH_VECTOR_LEN];
+	MD5_CTX		context;
 
-	memcpy(recvbuf + 4, original->vector, sizeof(original->vector));
-	secretlen = strlen(secret);
-	memcpy(recvbuf + len, secret, secretlen);
-	librad_md5_calc(calc_digest, (u_char *)recvbuf, len + secretlen);
+	/*
+	 *  Copy the original vector in place.
+	 */
+	memcpy(packet->data + 4, original->vector, sizeof(original->vector));
+
+	/*
+	 *  MD5(packet + secret);
+	 */
+	MD5Init(&context);
+	MD5Update(&context, packet->data, packet->data_len);
+	MD5Update(&context, secret, strlen(secret));
+	MD5Final(calc_digest, &context);
+
+	/*
+	 *  Copy the packet's vector back to the packet.
+	 */
+	memcpy(packet->data + 4, packet->vector, sizeof(packet->vector));
 
 	/*
 	 *	Return 0 if OK, 2 if not OK.
@@ -504,11 +614,16 @@ RADIUS_PACKET *rad_recv(int fd)
 	socklen_t		salen;
 	u_short			len;
 	uint8_t			*attr;
+	uint8_t			*vendorattr;
 	int			count;
 	radius_packet_t		*hdr;
 	char			host_ipaddr[16];
 	int			seen_eap;
-
+	uint8_t			data[4096];
+	int			num_attributes;
+	uint32_t                vendorcode;
+	int			vendorlen;
+	
 	/*
 	 *	Allocate the new request data structure
 	 */
@@ -518,19 +633,13 @@ RADIUS_PACKET *rad_recv(int fd)
 		return NULL;
 	}
 	memset(packet, 0, sizeof(RADIUS_PACKET));
-	if ((packet->data = malloc(PACKET_DATA_LEN)) == NULL) {
-		free(packet);
-		librad_log("out of memory");
-		errno = ENOMEM;
-		return NULL;
-	}
 
 	/*
 	 *	Receive the packet.
 	 */
 	salen = sizeof(saremote);
 	memset(&saremote, 0, sizeof(saremote));
-	packet->data_len = recvfrom(fd, packet->data, PACKET_DATA_LEN,
+	packet->data_len = recvfrom(fd, data, sizeof(data),
 		0, (struct sockaddr *)&saremote, &salen);
 
 	/*
@@ -553,7 +662,6 @@ RADIUS_PACKET *rad_recv(int fd)
 		librad_log("Error receiving packet from host %s: %s",
 			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
 			   strerror(errno));
-		free(packet->data);
 		free(packet);
 		return NULL;
 	}
@@ -562,9 +670,8 @@ RADIUS_PACKET *rad_recv(int fd)
 	 *	Check for packets smaller than the packet header.
 	 */
 	if (packet->data_len < AUTH_HDR_LEN) {
-		librad_log("Malformed RADIUS packet from host %s: too short",
+		librad_log("WARNING: Malformed RADIUS packet from host %s: too short",
 			   ip_ntoa(host_ipaddr, packet->src_ipaddr));
-		free(packet->data);
 		free(packet);
 		return NULL;
 	}
@@ -574,14 +681,13 @@ RADIUS_PACKET *rad_recv(int fd)
 	 *	i.e. We've received 128 bytes, and the packet header
 	 *	says it's 256 bytes long.
 	 */
-	hdr = (radius_packet_t *)packet->data;
+	hdr = (radius_packet_t *)data;
 	memcpy(&len, hdr->length, sizeof(u_short));
 	totallen = ntohs(len);
 	if (packet->data_len != totallen) {
-		librad_log("Malformed RADIUS packet from host %s: received %d octets, packet size says %d",
+		librad_log("WARNING: Malformed RADIUS packet from host %s: received %d octets, packet size says %d",
 			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
 			   packet->data_len, totallen);
-		free(packet->data);
 		free(packet);
 		return NULL;
 	}
@@ -601,15 +707,15 @@ RADIUS_PACKET *rad_recv(int fd)
 	attr = hdr->data;
 	count = totallen - AUTH_HDR_LEN;
 	seen_eap = 0;
+	num_attributes = 0;
 
 	while (count > 0) {
 		/*
 		 *	Attribute number zero is NOT defined.
 		 */
 		if (attr[0] == 0) {
-			librad_log("Malformed RADIUS packet from host %s: Invalid attribute 0",
+			librad_log("WARNING: Malformed RADIUS packet from host %s: Invalid attribute 0",
 				   ip_ntoa(host_ipaddr, packet->src_ipaddr));
-			free(packet->data);
 			free(packet);
 			return NULL;
 		}
@@ -619,10 +725,9 @@ RADIUS_PACKET *rad_recv(int fd)
 		 *	fields.  Anything shorter is an invalid attribute.
 		 */
        		if (attr[1] < 2) {
-			librad_log("Malformed RADIUS packet from host %s: attribute %d too short",
+			librad_log("WARNING: Malformed RADIUS packet from host %s: attribute %d too short",
 				   ip_ntoa(host_ipaddr, packet->src_ipaddr),
 				   attr[0]);
-			free(packet->data);
 			free(packet);
 			return NULL;
 		}
@@ -640,15 +745,56 @@ RADIUS_PACKET *rad_recv(int fd)
 
 		case PW_MESSAGE_AUTHENTICATOR:
 			if (attr[1] != 2 + AUTH_VECTOR_LEN) {
-				librad_log("Malformed RADIUS packet from host %s: Message-Authenticator has invalid length %d",
+				librad_log("WARNING: Malformed RADIUS packet from host %s: Message-Authenticator has invalid length %d",
 					   ip_ntoa(host_ipaddr, packet->src_ipaddr),
 					   attr[1] - 2);
-				free(packet->data);
 				free(packet);
 				return NULL;
 			}
 			seen_eap |= PW_MESSAGE_AUTHENTICATOR;
 			break;
+			
+		case PW_VENDOR_SPECIFIC:
+			if (attr[1] < 6) {
+				librad_log("WARNING: Malformed RADIUS packet from host %s: Vendor-Specific has invalid length %d",
+					   ip_ntoa(host_ipaddr, packet->src_ipaddr),
+					   attr[1] - 2);
+				free(packet);
+				return NULL;
+			}
+			memcpy(&vendorcode, attr + 2, 4);
+			vendorcode = ntohl(vendorcode);
+			if (vendorcode == VENDORPEC_USR) {
+				if (attr[1] < 8){
+					librad_log("WARNING: Malformed RADIUS packet from host %s: USR attribute has invalid length %d",
+					   ip_ntoa(host_ipaddr, packet->src_ipaddr),
+					   attr[1] - 2);
+					free(packet);
+					return NULL;
+				}
+				break;
+			}
+			vendorlen = attr[1] - 6;
+			vendorattr = attr + 6;
+			while (vendorlen >= 2) {
+				if (vendorattr[1] < 2){
+					librad_log("WARNING: Malformed RADIUS packet from host %s: Vendor specific attribute has invalid length %d",
+					   ip_ntoa(host_ipaddr, packet->src_ipaddr),
+					   vendorattr[1] - 2);
+					free(packet);
+					return NULL;
+				}
+				vendorlen -= vendorattr[1];
+				vendorattr += vendorattr[1];
+			}
+			if (vendorlen != 0){
+				librad_log("WARNING: Malformed RADIUS packet from host %s: Vendor specific attributes do not exactly fill Vendor-Specific",
+				   ip_ntoa(host_ipaddr, packet->src_ipaddr),
+				   vendorattr[1] - 2);
+				free(packet);
+				return NULL;
+			}
+			break;
 		}
 
 		/*
@@ -657,9 +803,9 @@ RADIUS_PACKET *rad_recv(int fd)
 		 *	integer/date/ip, the attribute length SHOULD
 		 *	be 6.
 		 */
-
 		count -= attr[1];	/* grab the attribute length */
 		attr += attr[1];
+		num_attributes++;	/* seen one more attribute */
 	}
 
 	/*
@@ -668,9 +814,8 @@ RADIUS_PACKET *rad_recv(int fd)
 	 *	If not, we complain, and throw the packet away.
 	 */
 	if (count != 0) {
-		librad_log("Malformed RADIUS packet from host %s: packet attributes do NOT exactly fill the packet",
+		librad_log("WARNING: Malformed RADIUS packet from host %s: packet attributes do NOT exactly fill the packet",
 			   ip_ntoa(host_ipaddr, packet->src_ipaddr));
-		free(packet->data);
 		free(packet);
 		return NULL;
 	}
@@ -681,9 +826,22 @@ RADIUS_PACKET *rad_recv(int fd)
 	 */
 	if (((seen_eap & PW_EAP_MESSAGE) == PW_EAP_MESSAGE) &&
 	    ((seen_eap & PW_MESSAGE_AUTHENTICATOR) != PW_MESSAGE_AUTHENTICATOR)) {
-		librad_log("Malformed RADIUS packet from host %s: Contains EAP-Message, but no Message-Authenticator",
+		librad_log("WARNING: Malformed RADIUS packet from host %s: Contains EAP-Message, but no Message-Authenticator",
 			   ip_ntoa(host_ipaddr, packet->src_ipaddr));
-		free(packet->data);
+		free(packet);
+		return NULL;
+	}
+
+	/*
+	 *	If we're configured to look for a maximum number of
+	 *	attributes, and we've seen more than that maximum,
+	 *	then throw the packet away, as a possible DoS.
+	 */
+	if ((librad_max_attributes > 0) &&
+	    (num_attributes > librad_max_attributes)) {
+		librad_log("WARNING: Possible DoS attack from host %s: Too many attributes in request (received %d, max %d are allowed).",
+			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
+			   num_attributes, librad_max_attributes);
 		free(packet);
 		return NULL;
 	}
@@ -708,6 +866,18 @@ RADIUS_PACKET *rad_recv(int fd)
 	packet->id = hdr->id;
 	memcpy(packet->vector, hdr->vector, AUTH_VECTOR_LEN);
 
+	/*
+	 *  Now that we've sanity checked the packet, we can allocate
+	 *  memory for it, and copy the data from the local area to
+	 *  the packet buffer.
+	 */
+	if ((packet->data = malloc(packet->data_len)) == NULL) {
+	  free(packet);
+	  librad_log("out of memory");
+	  return NULL;
+	}
+	memcpy(packet->data, data, packet->data_len);
+
 	return packet;
 }
 
@@ -719,7 +889,6 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 	DICT_ATTR		*attr;
 	uint32_t		lvalue;
 	uint32_t		vendorcode;
-	uint32_t		vendorpec;
 	VALUE_PAIR		*first_pair;
 	VALUE_PAIR		*prev;
 	VALUE_PAIR		*pair;
@@ -794,7 +963,6 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 	/*
 	 *	Calculate and/or verify digest.
 	 */
-	length = packet->data_len;
 	switch(packet->code) {
 		case PW_AUTHENTICATION_REQUEST:
 		case PW_STATUS_SERVER:
@@ -805,8 +973,7 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 			break;
 
 		case PW_ACCOUNTING_REQUEST:
-			if (calc_acctdigest(packet, secret,
-			    (char *)packet->data, length) > 1) {
+			if (calc_acctdigest(packet, secret) > 1) {
 				char buffer[32];
 				librad_log("Received Accounting-Request packet "
 				    "from %s with invalid signature!",
@@ -819,9 +986,7 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 		case PW_AUTHENTICATION_ACK:
 		case PW_AUTHENTICATION_REJECT:
 		case PW_ACCOUNTING_RESPONSE:
-			if (calc_replydigest(packet, original, secret,
-					     (char *)packet->data,
-					     length) > 1) {
+			if (calc_replydigest(packet, original, secret) > 1) {
 				char buffer[32];
 				librad_log("Received %s packet "
 					   "from %s with invalid signature!",
@@ -863,10 +1028,10 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 		    (attribute == PW_VENDOR_SPECIFIC) && 
 		    (attrlen > 6)) {
 			memcpy(&lvalue, ptr, 4);
-			vendorpec = ntohl(lvalue);
-			if ((vendorcode = dict_vendorcode(vendorpec)) != 0) {
+			vendorcode = ntohl(lvalue);
+			if (vendorcode != 0) {
 
-				if (vendorpec == VENDORPEC_USR) {
+				if (vendorcode == VENDORPEC_USR) {
 					ptr += 4;
 					memcpy(&lvalue, ptr, 4);
 					/*printf("received USR %04x\n", ntohl(lvalue));*/
@@ -907,6 +1072,7 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 		} else {
 			strcpy(pair->name, attr->name);
 			pair->type = attr->type;
+			pair->flags = attr->flags;
 		}
 		pair->attribute = attribute;
 		pair->length = attrlen;
@@ -931,9 +1097,32 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 				memcpy(pair->strvalue, my_digest, AUTH_VECTOR_LEN );
 				pair->strvalue[AUTH_VECTOR_LEN] = '\0';
 				pair->length = strlen(pair->strvalue);
-			} else
+			} else if (pair->flags.has_tag &&
+				   pair->type == PW_TYPE_STRING) {
+			        if(TAG_VALID(*ptr)) 
+				       pair->flags.tag = *ptr;
+				else
+				       pair->flags.tag = 0x00;
+				pair->length--;
+				memcpy(pair->strvalue, ptr + 1, 
+				       pair->length);
+			} else {
 				/* attrlen always < MAX_STRING_LEN */
 				memcpy(pair->strvalue, ptr, attrlen);
+			        pair->flags.tag = 0;
+			}
+
+			/* FIXME: Decrypt attributes like Tunnel-Password here -cparker */
+			if(pair->flags.encrypt == 2) {
+
+			        rad_tunnel_pwdecode((char *)pair->strvalue,
+						     pair->length, 
+						     secret,
+						     (char *)original->vector);
+				pair->length = strlen(pair->strvalue);
+
+			}
+
 			break;
 			
 		case PW_TYPE_INTEGER:
@@ -953,11 +1142,22 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 				pair->lvalue = 0xbaddbadd;
 				break;
 			}
-			memcpy(&lvalue, ptr, 4);
+
+      			memcpy(&lvalue, ptr, 4);
+
 			if (attr->type != PW_TYPE_IPADDR)
 				pair->lvalue = ntohl(lvalue);
 			else
 				pair->lvalue = lvalue;
+
+			/*
+			 *  Only PW_TYPE_INTEGER should have tags.
+			 */
+			if (pair->flags.has_tag &&
+			    pair->type == PW_TYPE_INTEGER) {
+			        pair->flags.tag = (pair->lvalue >> 24) & 0xff;
+				pair->lvalue &= 0xffffff;
+			}
 			break;
 			
 		default:
@@ -1108,6 +1308,142 @@ int rad_pwdecode(char *passwd, int pwlen, const char *secret, const char *vector
 	return pwlen;
 }
 
+/*
+ *	Encode Tunnel-Password attributes when sending them out on the wire.
+ *
+ *	int *pwlen is updated to the new length of the encrypted
+ *	password - a multiple of 16 bytes.
+ *
+ *      This is per RFC-2868 which adds a two char SALT to the initial intermediate
+ *      value MD5 hash.
+ */
+
+int rad_tunnel_pwencode(char *passwd, int *pwlen, const char *secret, const char *vector)
+{
+	uint8_t	buffer[AUTH_VECTOR_LEN + MAX_STRING_LEN + 3];
+	char	digest[AUTH_VECTOR_LEN];
+	char    salt[2];
+	int	i, n, secretlen;
+	int	len;
+
+	if(pwlen < 2) {
+	  return 0;
+	}
+	salt[0] = passwd[0];
+	salt[1] = passwd[1];
+
+	/* Advance pointer past the salt, which is first two chars of passwd */
+	passwd = passwd + 2;
+	
+	/*
+	 *	Padd password to multiple of AUTH_PASS_LEN bytes.
+	 */
+	len = strlen(passwd);
+	if (len > 128) len = 128;
+	*pwlen = len;
+	if (len % AUTH_PASS_LEN != 0) {
+		n = AUTH_PASS_LEN - (len % AUTH_PASS_LEN);
+		for (i = len; n > 0; n--, i++)
+			passwd[i] = 0;
+		len = *pwlen = i;
+	}
+
+	/*
+	 *	Use the secret to setup the decryption digest
+	 */
+	secretlen = strlen(secret);
+	memcpy(buffer, secret, secretlen);
+	memcpy(buffer + secretlen, vector, AUTH_VECTOR_LEN);
+	memcpy(buffer + secretlen + AUTH_VECTOR_LEN, salt, 2);
+	librad_md5_calc((u_char *)digest, buffer, secretlen + AUTH_VECTOR_LEN + 2);
+
+	/*
+	 *	Now we can encode the password *in place*
+	 */
+	for (i = 0; i < AUTH_PASS_LEN; i++)
+		passwd[i] ^= digest[i];
+
+	if (len <= AUTH_PASS_LEN) return 0;
+
+	/*
+	 *	Length > AUTH_PASS_LEN, so we need to use the extended
+	 *	algorithm.
+	 */
+	for (n = 0; n < 128 && n <= (len - AUTH_PASS_LEN); n += AUTH_PASS_LEN) { 
+		memcpy(buffer + secretlen, passwd + n, AUTH_PASS_LEN);
+		librad_md5_calc((u_char *)digest, buffer, secretlen + AUTH_PASS_LEN);
+		for (i = 0; i < AUTH_PASS_LEN; i++)
+			passwd[i + n + AUTH_PASS_LEN] ^= digest[i];
+	}
+
+	return 0;
+}
+
+/*
+ *	Decode Tunnel-Password encrypted attributes.
+ *
+ *      Defined in RFC-2868, this adds a two char SALT to the initial intermediate
+ *      value, to differentiate it from the above.
+ */
+
+int rad_tunnel_pwdecode(char *passwd, int pwlen, const char *secret, const char *vector)
+{
+	uint8_t	buffer[AUTH_VECTOR_LEN + MAX_STRING_LEN + 3];
+	char	digest[AUTH_VECTOR_LEN];
+	char	r[AUTH_VECTOR_LEN];
+	char    salt[2];
+	char	*s;
+	int	i, n, secretlen;
+	int	rlen;
+
+	if(pwlen < 2) {
+	  return 0;
+	}
+	salt[0] = passwd[0];
+	salt[1] = passwd[1];
+
+	passwd = passwd + 2;
+        pwlen = pwlen - 2;
+
+	/*
+	 *	Use the secret to setup the decryption digest
+	 */
+	secretlen = strlen(secret);
+	memcpy(buffer, secret, secretlen);
+	memcpy(buffer + secretlen, vector, AUTH_VECTOR_LEN);
+	memcpy(buffer + secretlen + AUTH_VECTOR_LEN, salt, 2);
+	librad_md5_calc((u_char *)digest, buffer, secretlen + AUTH_VECTOR_LEN + 2);
+
+	/*
+	 *	Now we can decode the password *in place*
+	 */
+	memcpy(r, passwd, AUTH_PASS_LEN);
+	for (i = 0; i < AUTH_PASS_LEN && i < pwlen; i++)
+		passwd[i] ^= digest[i];
+
+	if (pwlen <= AUTH_PASS_LEN) {
+		passwd[pwlen+1] = '\0';
+		return pwlen;
+	}
+
+	/*
+	 *	Length > AUTH_PASS_LEN, so we need to use the extended
+	 *	algorithm.
+	 */
+	rlen = ((pwlen - 1) / AUTH_PASS_LEN) * AUTH_PASS_LEN;
+
+	for (n = rlen; n > 0; n -= AUTH_PASS_LEN ) { 
+		s = (n == AUTH_PASS_LEN) ? r : (passwd + n - AUTH_PASS_LEN);
+		memcpy(buffer + secretlen, s, AUTH_PASS_LEN);
+		librad_md5_calc((u_char *)digest, buffer, secretlen + AUTH_PASS_LEN);
+		for (i = 0; i < AUTH_PASS_LEN && (i + n) < pwlen; i++)
+			passwd[i + n] ^= digest[i];
+	}
+	passwd[pwlen] = '\0';
+
+	return pwlen;
+}
+
 /*
  *	Encode a CHAP password
  *
