@@ -1,11 +1,11 @@
 /*
  * valuepair.c	Functions to handle VALUE_PAIRs
  *
- * Version:	$Id: valuepair.c,v 1.42 2001/09/06 16:14:43 aland Exp $
+ * Version:	$Id: valuepair.c,v 1.45 2001/11/01 00:06:43 cparker Exp $
  *
  */
 
-static const char rcsid[] = "$Id: valuepair.c,v 1.42 2001/09/06 16:14:43 aland Exp $";
+static const char rcsid[] = "$Id: valuepair.c,v 1.45 2001/11/01 00:06:43 cparker Exp $";
 
 #include	"autoconf.h"
 
@@ -379,10 +379,21 @@ void pairmove2(VALUE_PAIR **to, VALUE_PAIR **from, int attr)
 
 
 		/*
-		 *	FIXME: Do we want to match *any* VSA if
-		 *	'attr == PW_VENDOR_SPECIFIC' ???
+		 *	If the attribute to move is NOT a VSA, then it
+		 *	ignores any attributes which do not match exactly.
 		 */
-		if (i->attribute != attr) {
+		if ((attr != PW_VENDOR_SPECIFIC) &&
+		    (i->attribute != attr)) {
+			iprev = i;
+			continue;
+		}
+
+		/*
+		 *	If the attribute to move IS a VSA, then it ignores
+		 *	any non-VSA attribute.
+		 */
+		if ((attr == PW_VENDOR_SPECIFIC) &&
+		    (VENDOR(i->attribute) == 0)) {
 			iprev = i;
 			continue;
 		}
@@ -488,6 +499,32 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 	VALUE_PAIR	*vp;
 	char		*p, *s=0;
 	const char	*cp, *cs;
+	char            *tc, *ts;
+	signed char     tag;
+	int             found_tag;
+
+	/*
+	 *    Check for tags in 'Attribute:Tag' format.
+	 */
+	found_tag = 0;
+	tag = 0;
+
+	ts = strrchr( attribute, ':' );
+	if (ts && ts[1]) {
+	         /* Colon found with something behind it */
+	         if (ts[1] == '*' && ts[2] == 0) {
+		         /* Wildcard tag for check items */
+		         tag = TAG_ANY;
+			 *ts = 0;
+		 } else {
+		         /* It's not a wild card tag */
+		         tag = strtol(ts + 1, &tc, 0);
+			 if (tc && !*tc && TAG_VALID_ZERO(tag))
+			        *ts = 0;
+			 else tag = 0;
+		 }
+		 found_tag = 1;
+	}
 
 	if ((da = dict_attrbyname(attribute)) == NULL) {
 		librad_log("Unknown attribute %s", attribute);
@@ -504,8 +541,40 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 	vp->type = da->type;
 	vp->operator = (operator == 0) ? T_OP_EQ : operator;
 	strcpy(vp->name, da->name);
+	vp->flags = da->flags;
 	vp->next = NULL;
 
+	/*      Check for a tag in the 'Merit' format of:
+	 *      :Tag:Value.  Print an error if we already found
+	 *      a tag in the Attribute.
+	 */
+
+	if (*value == ':' && da->flags.has_tag) {
+	        /* If we already found a tag, this is invalid */
+	        if(found_tag) {
+		        free(vp);
+			librad_log("Duplicate tag %s for attribute %s",
+				   value, vp->name);
+			DEBUG("Duplicate tag %s for attribute %s\n",
+				   value, vp->name);
+			return NULL;
+
+		}
+	        /* Colon found and attribute allows a tag */
+	        if (value[1] == '*' && value[2] == ':') {
+		       /* Wildcard tag for check items */
+		       tag = TAG_ANY;
+		       value += 3;
+		} else {
+	               /* Real tag */
+		       tag = strtol(value + 1, &tc, 0);
+		       if (tc && *tc==':' && TAG_VALID_ZERO(tag))
+			    value = tc + 1;
+		       else tag = 0;
+		}
+		found_tag = 1;
+	}	
+	
 	/*
 	 *	Even for integers, dates and ip addresses we
 	 *	keep the original string in vp->strvalue.
@@ -528,7 +597,7 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 				cs = s = strdup(value);
 				p = strrchr(s, '+');
 				*p = 0;
-				vp->addport = 1;
+				vp->flags.addport = 1;
 			} else {
 				p = NULL;
 				cs = value;
@@ -616,6 +685,9 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 			librad_log("unknown attribute type %d", da->type);
 			return NULL;
 	}
+	if(found_tag) {
+	  vp->flags.tag = tag;
+	}
 	return vp;
 }
 
@@ -623,7 +695,7 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
  *	Read a valuepair from a buffer, and advance pointer.
  *	Sets *eol to T_EOL if end of line was encountered.
  */
-VALUE_PAIR *pairread(char **ptr, int *eol)
+VALUE_PAIR *pairread(char **ptr, LRAD_TOKEN *eol)
 {
 	char		buf[64];
 	char		attr[64];
