@@ -1,7 +1,7 @@
 /*
  * auth.c	User authentication.
  *
- * Version:	$Id: auth.c,v 1.90 2001/09/17 19:25:43 aland Exp $
+ * Version:	$Id: auth.c,v 1.95 2001/10/30 17:22:04 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  */
 
-static const char rcsid[] = "$Id: auth.c,v 1.90 2001/09/17 19:25:43 aland Exp $";
+static const char rcsid[] = "$Id: auth.c,v 1.95 2001/10/30 17:22:04 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -113,19 +113,29 @@ static int rad_authlog(const char *msg, REQUEST *request, int goodpass) {
 	char clean_password[1024];
 	char clean_username[1024];
 	char buf[1024];
+	VALUE_PAIR *username = NULL;
 
 	if (!mainconfig.log_auth) {
 		return 0;
 	}
 
+	/*
+	 * Get the correct username based on the configured value
+	 */
+	if (log_stripped_names == 0) {
+		username = pairfind(request->packet->vps, PW_USER_NAME);
+	} else {
+		username = request->username;
+	}
+
 	/* 
 	 *	Clean up the username
 	 */
-	if (request->username == NULL) {
+	if (username == NULL) {
 		strcpy(clean_username, "<no User-Name attribute>");
 	} else {
-		librad_safeprint((char *)request->username->strvalue,
-				request->username->length,
+		librad_safeprint((char *)username->strvalue,
+				username->length,
 				clean_username, sizeof(clean_username));
 	}
 
@@ -242,16 +252,6 @@ int rad_check_password(REQUEST *request)
 		return 0;
 	}
 
-	/*
-	 *	Find the password sent by the user. It SHOULD be there,
-	 *	if it's not authentication fails.
-	 */
-	auth_item = request->password;
-	if (auth_item == NULL) {
-		DEBUG2("auth: No Password or CHAP-Password attribute in the request");
-		return -1;
-	}
-
 	/*
 	 *	Find the password from the users file.
 	 */
@@ -290,6 +290,17 @@ int rad_check_password(REQUEST *request)
 
 	switch(auth_type) {
 		case PW_AUTHTYPE_CRYPT:
+			/*
+			 *	Find the password sent by the user. It
+			 *	SHOULD be there, if it's not
+			 *	authentication fails.
+			 */
+			auth_item = request->password;
+			if (auth_item == NULL) {
+				DEBUG2("auth: No Password or CHAP-Password attribute in the request");
+				return -1;
+			}
+
 			DEBUG2("auth: type Crypt");
 			if (password_pair == NULL) {
 				DEBUG2("No Crypt-Password configured for the user");
@@ -305,6 +316,17 @@ int rad_check_password(REQUEST *request)
 		case PW_AUTHTYPE_LOCAL:
 			DEBUG2("auth: type Local");
 
+			/*
+			 *	Find the password sent by the user. It
+			 *	SHOULD be there, if it's not
+			 *	authentication fails.
+			 */
+			auth_item = request->password;
+			if (auth_item == NULL) {
+				DEBUG2("auth: No Password or CHAP-Password attribute in the request");
+				return -1;
+			}
+
 			/*
 			 *	Plain text password.
 			 */
@@ -393,7 +415,7 @@ int rad_authenticate(REQUEST *request)
 	VALUE_PAIR	*check_item;
 	VALUE_PAIR	*reply_item;
 	VALUE_PAIR	*auth_item;
-	VALUE_PAIR	*tmp;
+	VALUE_PAIR	*tmp = NULL;
 	int		result, r;
 	char		umsg[MAX_STRING_LEN + 1];
 	const char	*user_msg = NULL;
@@ -505,7 +527,17 @@ int rad_authenticate(REQUEST *request)
 			r != RLM_MODULE_OK &&
 			r != RLM_MODULE_UPDATED) {
 		if (r != RLM_MODULE_FAIL && r != RLM_MODULE_HANDLED) {
-			rad_authlog("Invalid user", request, 0);
+			VALUE_PAIR *module_msg;
+
+			if ((module_msg = pairfind(request->packet->vps,
+					PW_MODULE_MESSAGE)) != NULL){
+				char msg[MAX_STRING_LEN+16];
+				snprintf(msg, sizeof(msg), "Invalid user (%s)",
+					 module_msg->strvalue);
+				rad_authlog(msg,request,0);
+			} else {
+				rad_authlog("Invalid user", request, 0);
+			}
 			request->reply->code = PW_AUTHENTICATION_REJECT;
 		}
 		/*
@@ -552,11 +584,20 @@ int rad_authenticate(REQUEST *request)
 	 *	wants to send back.
 	 */
 	if (result < 0) {
+		VALUE_PAIR *module_msg;
 
 		DEBUG2("auth: Failed to validate the user.");
 		request->reply->code = PW_AUTHENTICATION_REJECT;
 		
-		rad_authlog("Login incorrect", request, 0);
+		if ((module_msg = pairfind(request->packet->vps,PW_MODULE_MESSAGE)) != NULL){
+			char msg[MAX_STRING_LEN+19];
+
+			snprintf(msg, sizeof(msg), "Login incorrect (%s)",
+				 module_msg->strvalue);
+			rad_authlog(msg, request, 0);
+		} else {
+			rad_authlog("Login incorrect", request, 0);
+		}
 
 		/* double check: maybe the secret is wrong? */
 		if ((debug_flag > 1) && (auth_item != NULL) &&
@@ -582,32 +623,45 @@ int rad_authenticate(REQUEST *request)
 		 */
 		if (namepair &&
 				(r = module_checksimul(request, check_item->lvalue)) != 0) {
+			char mpp_ok = 0;
 
-			if (check_item->lvalue > 1) {
-			  snprintf(umsg, sizeof(umsg), 
-						"\r\nYou are already logged in %d times  - access denied\r\n\n",
-						(int)check_item->lvalue);
-				user_msg = umsg;
-			} else {
-				user_msg = "\r\nYou are already logged in - access denied\r\n\n";
+			if (r == 2){
+				/* Multilink attempt. Check if port-limit > simultaneous-use */
+				VALUE_PAIR *port_limit;
+
+				if ((port_limit = pairfind(request->reply->vps, PW_PORT_LIMIT)) != NULL &&
+					port_limit->lvalue > check_item->lvalue){
+					DEBUG2("main auth: MPP is OK");
+					mpp_ok = 1;
+				}
 			}
+			if (!mpp_ok){
+				if (check_item->lvalue > 1) {
+		  		snprintf(umsg, sizeof(umsg), 
+							"\r\nYou are already logged in %d times  - access denied\r\n\n",
+							(int)check_item->lvalue);
+					user_msg = umsg;
+				} else {
+					user_msg = "\r\nYou are already logged in - access denied\r\n\n";
+				}
 
-			request->reply->code = PW_AUTHENTICATION_REJECT;
+				request->reply->code = PW_AUTHENTICATION_REJECT;
 
-			/*
-			 *	They're trying to log in too many times.
-			 *	Remove ALL reply attributes.
-			 */
-			pairfree(&request->reply->vps);
-			tmp = pairmake("Reply-Message", user_msg, T_OP_SET);
-			request->reply->vps = tmp;
+				/*
+		 		*	They're trying to log in too many times.
+		 		*	Remove ALL reply attributes.
+		 		*/
+				pairfree(&request->reply->vps);
+				tmp = pairmake("Reply-Message", user_msg, T_OP_SET);
+				request->reply->vps = tmp;
 
-			snprintf(logstr, sizeof(logstr), "Multiple logins (max %d) %s",
-				check_item->lvalue,
-				r == 2 ? "[MPP attempt]" : "");
-			rad_authlog(logstr, request, 1);
+				snprintf(logstr, sizeof(logstr), "Multiple logins (max %d) %s",
+					check_item->lvalue,
+					r == 2 ? "[MPP attempt]" : "");
+				rad_authlog(logstr, request, 1);
 
-			result = -1;
+				result = -1;
+			}
 		}
 	}
 
@@ -686,7 +740,7 @@ int rad_authenticate(REQUEST *request)
 	 */
 	if (((tmp = pairfind(request->reply->vps, 
 			    PW_FRAMED_IP_ADDRESS)) != NULL) &&
-	    (tmp->addport != 0)) {
+	    (tmp->flags.addport != 0)) {
 		VALUE_PAIR *vpPortId;
 		
 		/*
@@ -695,7 +749,7 @@ int rad_authenticate(REQUEST *request)
 		if ((vpPortId = pairfind(request->packet->vps,
 					 PW_NAS_PORT_ID)) != NULL) {
 		  tmp->lvalue = htonl(ntohl(tmp->lvalue) + vpPortId->lvalue);
-		  tmp->addport = 0;
+		  tmp->flags.addport = 0;
 		}
 	}
 
