@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: conffile.c,v 1.74 2001/09/20 18:09:36 aland Exp $
+ * Version:	$Id: conffile.c,v 1.78 2001/11/08 16:58:56 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 #include "modules.h"
 
 static const char rcsid[] =
-"$Id: conffile.c,v 1.74 2001/09/20 18:09:36 aland Exp $";
+"$Id: conffile.c,v 1.78 2001/11/08 16:58:56 aland Exp $";
 
 #define xstrdup strdup
 
@@ -234,8 +234,8 @@ static void cf_item_add(CONF_SECTION *cs, CONF_ITEM *ci_new)
  *	Expand the variables in an input string.
  */
 static const char *cf_expand_variables(const char *cf, int *lineno,
-		CONF_SECTION *cs,
-		char *output, const char *input)
+				       CONF_SECTION *cs,
+				       char *output, const char *input)
 {
 	char *p;
 	const char *end, *ptr;
@@ -245,7 +245,7 @@ static const char *cf_expand_variables(const char *cf, int *lineno,
 
 	p = output;
 	ptr = input;
-	while (*ptr >= ' ') {
+	while (*ptr) {
 		/*
 		 *	Ignore anything other than "${"
 		 */
@@ -857,36 +857,58 @@ static int generate_realms(const char *filename)
 		 */
 		c = rad_malloc(sizeof(REALM));
 		memset(c, 0, sizeof(REALM));
+
+		c->secret[0] = '\0';
+
 		/*
-		 * An authhost must exist in the configuration
+		 *	No authhost means LOCAL.
 		 */
 		if ((authhost = cf_section_value_find(cs, "authhost")) == NULL) {
-			radlog(L_CONS|L_ERR, 
-				"%s[%d]: No authhost entry for realm: %s", 
-				filename, cs->item.lineno, cs->name2);
-			return -1;
-		}
-		if ((s = strchr(authhost, ':')) != NULL) {
-			*s++ = 0;
-			c->auth_port = atoi(s);
-		} else {
+			c->ipaddr = htonl(INADDR_NONE);
 			c->auth_port = auth_port;
-		}
-		if ((accthost = cf_section_value_find(cs, "accthost")) == NULL) {
-			radlog(L_CONS|L_ERR, 
-				"%s[%d]: No accthost entry for realm: %s", 
-				filename, cs->item.lineno, cs->name2);
-			return -1;
+		} else {
+			if ((s = strchr(authhost, ':')) != NULL) {
+				*s++ = 0;
+				c->auth_port = atoi(s);
+			} else {
+				c->auth_port = auth_port;
+			}
+			if (strcmp(authhost, "LOCAL") == 0) {
+				/*
+				 *	Local realms don't have an IP address,
+				 *	secret, or port.
+				 */
+				c->ipaddr = htonl(INADDR_NONE);
+				c->auth_port = auth_port;
+			} else {
+				c->ipaddr = ip_getaddr(authhost);
+			}
 		}
 
-		if ((s =strchr(accthost, ':')) != NULL) {
-			*s++ = 0;
-			c->acct_port = atoi(s);	
-		} else {
+		/*
+		 *	No accthost means LOCAL
+		 */
+		if ((accthost = cf_section_value_find(cs, "accthost")) == NULL) {
+			c->acct_ipaddr = htonl(INADDR_NONE);
 			c->acct_port = acct_port;
+		} else {
+			if ((s = strchr(accthost, ':')) != NULL) {
+				*s++ = 0;
+				c->acct_port = atoi(s);	
+			} else {
+				c->acct_port = acct_port;
+			}
+			if (strcmp(accthost, "LOCAL") == 0) {
+				/*
+				 *	Local realms don't have an IP address,
+				 *	secret, or port.
+				 */
+				c->acct_ipaddr = htonl(INADDR_NONE);
+				c->acct_port = acct_port;
+			} else {
+				c->acct_ipaddr = ip_getaddr(authhost);
+			}
 		}
-		if (strcmp(authhost, "LOCAL") != 0)
-			c->ipaddr = ip_getaddr(authhost);
 
 		/* 
 		 * Double check length, just to be sure!
