@@ -1,7 +1,7 @@
 /*
  * files.c	Read config files into memory.
  *
- * Version:     $Id: files.c,v 1.56 2001/09/20 18:22:38 aland Exp $
+ * Version:     $Id: files.c,v 1.58 2001/11/05 15:18:51 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: files.c,v 1.56 2001/09/20 18:22:38 aland Exp $";
+static const char rcsid[] = "$Id: files.c,v 1.58 2001/11/05 15:18:51 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -374,7 +374,6 @@ int read_realms_file(const char *file)
 	char *s, *p;
 	int lineno = 0;
 	REALM *c, **tail;
-	RADCLIENT *client;
 
 	realm_free(realms);
 	realms = NULL;
@@ -415,28 +414,38 @@ int read_realms_file(const char *file)
 		}
 
 		if (strcmp(hostnm, "LOCAL") == 0) {
-			c->ipaddr = htonl(INADDR_LOOPBACK);
+			/*
+			 *	Local realms don't have an IP address,
+			 *	secret, or port.
+			 */
+			c->acct_ipaddr = c->ipaddr = htonl(INADDR_NONE);
+			c->secret[0] = '\0';
+			c->auth_port = auth_port;
+			c->acct_port = acct_port;
+
 		} else {
+			RADCLIENT *client;
 			c->ipaddr = ip_getaddr(hostnm);
-		}
+			c->acct_ipaddr = c->ipaddr;
 
-		if (c->ipaddr == 0) {
-			radlog(L_CONS|L_ERR, "%s[%d]: Failed to look up hostname %s",
-					file, lineno, hostnm);
-			return -1;
-		}
+			if (c->ipaddr == htonl(INADDR_NONE)) {
+				radlog(L_CONS|L_ERR, "%s[%d]: Failed to look up hostname %s",
+				       file, lineno, hostnm);
+				return -1;
+			}
 
-		/*
-		 *	Find the remote server in the "clients" list.
-		 *	If we can't find it, there's a big problem...
-		 */
-		client = client_find(c->ipaddr);
-		if (client == NULL) {
-			radlog(L_CONS|L_ERR, "%s[%d]: Cannot find 'clients' file entry of remote server %s for realm \"%s\"",
-					file, lineno, hostnm, realm);
-			return -1;
+			/*
+			 *	Find the remote server in the "clients" list.
+			 *	If we can't find it, there's a big problem...
+			 */
+			client = client_find(c->ipaddr);
+			if (client == NULL) {
+			  radlog(L_CONS|L_ERR, "%s[%d]: Cannot find 'clients' file entry of remote server %s for realm \"%s\"",
+				 file, lineno, hostnm, realm);
+			  return -1;
+			}
+			memcpy(c->secret, client->secret, sizeof(c->secret));
 		}
-		memcpy(c->secret, client->secret, sizeof(c->secret));
 
 		/*
 		 *	Double-check lengths to be sure they're sane
@@ -495,7 +504,8 @@ void realm_disable(uint32_t ipaddr)
 
 	now = time(NULL);
 	for(cl = realms; cl; cl = cl->next)
-		if (ipaddr == cl->ipaddr) {
+		if ((ipaddr == cl->ipaddr) ||
+		    (ipaddr == cl->acct_ipaddr)) {
 			cl->active = FALSE;
 			cl->wakeup = now + proxy_dead_time;
 		}
@@ -573,7 +583,8 @@ REALM *realm_findbyaddr(uint32_t ipaddr)
 	 *	as active, and return the realm.
 	 */
 	for(cl = realms; cl != NULL; cl = cl->next)
-		if (ipaddr == cl->ipaddr) {
+		if ((ipaddr == cl->ipaddr) ||
+		    (ipaddr == cl->acct_ipaddr)) {
 			cl->active = TRUE;
 			return cl;
 		}
