@@ -1,7 +1,7 @@
 /*
  * proxy.c	Proxy stuff.
  *
- * Version:	$Id: proxy.c,v 1.48 2001/08/29 15:00:06 aland Exp $
+ * Version:	$Id: proxy.c,v 1.51 2001/11/06 16:41:49 cparker Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Chris Parker <cparker@starnetusa.com>
  */
 
-static const char rcsid[] = "$Id: proxy.c,v 1.48 2001/08/29 15:00:06 aland Exp $";
+static const char rcsid[] = "$Id: proxy.c,v 1.51 2001/11/06 16:41:49 cparker Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -180,12 +180,13 @@ int proxy_send(REQUEST *request)
 	
 
 	/*
-	 *	Maybe they're proxying it to a LOCAL realm, in which
-	 *	case do nothing.
+	 *	Access-Request: look for LOCAL realm.
+	 *	Accounting-Request: look for LOCAL realm.
 	 */
-	if ((realm->ipaddr == htonl(INADDR_LOOPBACK)) &&
-			(realm->auth_port == auth_port) &&
-			(realm->acct_port == acct_port)) {
+	if (((request->packet->code == PW_AUTHENTICATION_REQUEST) &&
+	     (realm->ipaddr == htonl(INADDR_NONE))) ||
+	    ((request->packet->code == PW_ACCOUNTING_REQUEST) &&	    
+	     (realm->acct_ipaddr == htonl(INADDR_NONE)))) {
 		return 0;
 	}
 	
@@ -249,11 +250,13 @@ int proxy_send(REQUEST *request)
 	request->proxy->sockfd = proxyfd;
 
 	request->proxy->code = request->packet->code;
-	request->proxy->dst_ipaddr = realm->ipaddr;
-	if (request->packet->code == PW_AUTHENTICATION_REQUEST)
+	if (request->packet->code == PW_AUTHENTICATION_REQUEST) {
 		request->proxy->dst_port = realm->auth_port;
-	else
+		request->proxy->dst_ipaddr = realm->ipaddr;
+	} else {
 		request->proxy->dst_port = realm->acct_port;
+		request->proxy->dst_ipaddr = realm->acct_ipaddr;
+	}
 	rad_assert(request->proxy->vps == NULL);
 	request->proxy->vps = vps;
 
