@@ -1,7 +1,7 @@
 /*
  * radiusd.c	Main loop of the radius server.
  *
- * Version:	$Id: radiusd.c,v 1.207 2001/07/24 16:04:13 aland Exp $
+ * Version:	$Id: radiusd.c,v 1.231 2001/11/20 17:13:22 cmiller Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 /* don't look here for the version, run radiusd -v or look in version.c */
 static const char rcsid[] =
-"$Id: radiusd.c,v 1.207 2001/07/24 16:04:13 aland Exp $";
+"$Id: radiusd.c,v 1.231 2001/11/20 17:13:22 cmiller Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -73,9 +73,8 @@ static const char rcsid[] =
 #	define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
 #endif
 
-#include <assert.h>
-
 #include "radiusd.h"
+#include "rad_assert.h"
 #include "conffile.h"
 #include "modules.h"
 #include "request_list.h"
@@ -96,10 +95,11 @@ const char *progname = NULL;
 const char *radius_dir = NULL;
 const char *radacct_dir = NULL;
 const char *radlog_dir = NULL;
+radlog_dest_t radlog_dest = RADLOG_FILES;
 const char *radlib_dir = NULL;
 int syslog_facility;
 int log_stripped_names;
-int debug_flag;
+int debug_flag = 0;
 int use_dbm = FALSE;
 uint32_t myip = INADDR_ANY;
 int log_auth_detail = FALSE;
@@ -108,7 +108,8 @@ int acct_port;
 int proxy_port;
 int proxy_retry_delay = RETRY_DELAY;
 int proxy_retry_count = RETRY_COUNT;
-int proxy_synchronous = TRUE;
+int proxy_dead_time = DEAD_TIME;
+int proxy_synchronous = FALSE;
 int need_reload = FALSE;
 struct main_config_t mainconfig;
 const char *radiusd_version = "FreeRADIUS Version " RADIUSD_VERSION ", for host " HOSTINFO ", built on " __DATE__ " at " __TIME__;
@@ -144,12 +145,14 @@ static void sig_fatal (int);
 static void sig_hup (int);
 
 static int switch_users(void);
+static void rfc_clean(RADIUS_PACKET *packet);
 static void rad_reject(REQUEST *request);
 static struct timeval *rad_clean_list(time_t curtime);
 static REQUEST *rad_check_list(REQUEST *);
 static REQUEST *proxy_check_list(REQUEST *request);
 static int refresh_request(REQUEST *request, void *data);
-#ifndef WITH_THREAD_POOL
+static int debug_level;		/* for configuration file stuff */
+#ifndef HAVE_PTHREAD_H
 static int rad_spawn_child(REQUEST *, RAD_REQUEST_FUNP);
 #else
 extern int rad_spawn_child(REQUEST *, RAD_REQUEST_FUNP);
@@ -161,7 +164,8 @@ extern int rad_spawn_child(REQUEST *, RAD_REQUEST_FUNP);
 static CONF_PARSER proxy_config[] = {
 	{ "retry_delay",  PW_TYPE_INTEGER, 0, &proxy_retry_delay, Stringify(RETRY_DELAY) },
 	{ "retry_count",  PW_TYPE_INTEGER, 0, &proxy_retry_count, Stringify(RETRY_COUNT) },
-	{ "synchronous",  PW_TYPE_BOOLEAN, 0, &proxy_synchronous, "yes" },
+	{ "synchronous",  PW_TYPE_BOOLEAN, 0, &proxy_synchronous, "no" },
+	{ "dead_time",    PW_TYPE_INTEGER, 0, &proxy_dead_time, Stringify(DEAD_TIME) },
 	{ NULL, -1, 0, NULL, NULL }
 };
 
@@ -190,6 +194,7 @@ static CONF_PARSER server_config[] = {
 	{ "nospace_pass", PW_TYPE_STRING_PTR, 0, &mainconfig.do_nospace_pass, "no" },
 	{ "proxy_requests", PW_TYPE_BOOLEAN, 0, &proxy_requests, "yes" },
 	{ "proxy", PW_TYPE_SUBSECTION, 0, proxy_config, NULL },
+	{ "debug_level", PW_TYPE_INTEGER, 0, &debug_level, "0"},
 	{ NULL, -1, 0, NULL, NULL }
 };
 
@@ -254,6 +259,7 @@ static int reread_config(int reload)
 	int pid = getpid();
 	CONF_SECTION *cs;
 	struct rlimit core_limits;
+	static int old_debug_level = -1;
 
 	if (!reload) {
 		radlog(L_INFO, "Starting - reading configuration files ...");
@@ -276,6 +282,47 @@ static int reread_config(int reload)
 	}
 	cf_section_parse(cs, NULL, server_config);
 
+	/*
+	 *	Set the libraries debugging flag to whatever the main
+	 *	flag is.  Note that on a SIGHUP, to turn the debugging
+	 *	off, we do other magic.
+	 *
+	 *	Increase the debug level, if the configuration file
+	 *	says to, OR, if we're decreasing the debug from what it
+	 *	was before, allow that, too.
+	 */
+	if ((debug_level > debug_flag) ||
+	    (debug_level <= old_debug_level)) {
+	  debug_flag = debug_level;
+	}
+	librad_debug = debug_flag;
+	old_debug_level = debug_level;
+	
+	/*
+	 *	If we're NOT debugging, trap fatal signals, so we can
+	 *	easily clean up after ourselves.
+	 *
+	 *	If we ARE debugging, don't trap them, so we can
+	 *	dump core.
+	 */
+	if ((allow_core_dumps == FALSE) && (debug_flag == 0)) {
+#ifdef SIGTRAP
+		signal(SIGTRAP, sig_fatal);
+#endif
+#ifdef SIGIOT
+		signal(SIGIOT, sig_fatal);
+#endif
+#ifdef SIGFPE
+		signal(SIGFPE, sig_fatal);
+#endif
+#ifdef SIGSEGV
+		signal(SIGSEGV, sig_fatal);
+#endif
+#ifdef SIGILL
+		signal(SIGILL, sig_fatal);
+#endif
+	}
+
 	/*  Reload the modules.  */
 	DEBUG2("read_config_files:  entering modules setup");
 	if (setup_modules() < 0) {
@@ -305,7 +352,7 @@ static int reread_config(int reload)
 			 *  dumps are enabled, log that information.
 			 */
 		} else if ((core_limits.rlim_cur != 0) && !debug_flag)
-			radlog(L_INFO, "Core dumps are enabled.");
+			radlog(L_INFO|L_CONS, "Core dumps are enabled.");
 
 	} else if (!debug_flag) {
 		/*
@@ -470,6 +517,7 @@ int main(int argc, char *argv[])
 	struct timeval *tv = NULL;
 
 	syslog_facility = LOG_DAEMON;
+
 #ifdef OSFC2
 	set_auth_parameters(argc,argv);
 #endif
@@ -495,23 +543,12 @@ int main(int argc, char *argv[])
 	signal(SIGINT, sig_fatal);
 	signal(SIGQUIT, sig_fatal);
 	signal(SIGPIPE, SIG_IGN);
-#ifdef SIGTRAP
-	signal(SIGTRAP, sig_fatal);
-#endif
-#ifdef SIGIOT
-	signal(SIGIOT, sig_fatal);
-#endif
+	signal(SIGTERM, sig_fatal);
 
 	/* this is right for threads, too, right?  
 	 * (Threads shouldn't get signals (from us) -- just be pthread_cancel()led.)
 	 */
-	signal(SIGTERM, sig_fatal);
 	signal(SIGCHLD, sig_cleanup);
-#if 0
-	signal(SIGFPE, sig_fatal);
-	signal(SIGSEGV, sig_fatal);
-	signal(SIGILL, sig_fatal);
-#endif
 
 	/*  Close unused file descriptors.  */
 	for (t = 32; t >= 3; t--)
@@ -528,19 +565,16 @@ int main(int argc, char *argv[])
 				break;
 
 			case 'a':
+				if (radacct_dir) free(radacct_dir);
 				radacct_dir = strdup(optarg);
 				break;
 			
-#if defined(WITH_DBM) || defined(WITH_NDBM)
-			case 'b':
-				use_dbm++;
-				break;
-#endif
 			case 'c':
 				/* ignore for backwards compatibility with Cistron */
 				break;
 
 			case 'd':
+				if (radius_dir) free(radius_dir);
 				radius_dir = strdup(optarg);
 				break;
 			
@@ -596,7 +630,6 @@ int main(int argc, char *argv[])
 				spawn_flag = FALSE;
 				dont_fork = TRUE;
 				debug_flag = 2;
-				librad_debug = 2;
 				mainconfig.log_auth = TRUE;
 				mainconfig.log_auth_badpass = TRUE;
 				mainconfig.log_auth_goodpass = TRUE;
@@ -605,7 +638,6 @@ int main(int argc, char *argv[])
 
 			case 'x':
 				debug_flag++;
-				librad_debug++;
 				break;
 			
 			case 'y':
@@ -644,11 +676,17 @@ int main(int argc, char *argv[])
 	 *  Also, initialize the logging facility with the
 	 *  configuration that they asked for.
 	 */
-	if (!strcmp(radlog_dir, "syslog")) {
+	if (strcmp(radlog_dir, "syslog") == 0) {
 		openlog(progname, LOG_PID, syslog_facility);
+		radlog_dest = RADLOG_SYSLOG;
 	}
 	/* Do you want a warning if -g is used without a -l to activate it? */
 #endif
+	if (strcmp(radlog_dir, "stdout") == 0) {
+		radlog_dest = RADLOG_STDOUT;
+	} else if (strcmp(radlog_dir, "stderr") == 0) {
+		radlog_dest = RADLOG_STDERR;
+	}
 
 	/*  Initialize the request list.  */
 	rl_init();
@@ -855,7 +893,7 @@ int main(int argc, char *argv[])
 		}
 	}
 
-#if WITH_THREAD_POOL
+#if HAVE_PTHREAD_H
 	/*
 	 *  If we're spawning children, set up the thread pool.
 	 */
@@ -887,6 +925,14 @@ int main(int argc, char *argv[])
 
 	radlog(L_INFO, "Ready to process requests.");
 
+
+	/* if we're running as a daemon, close other file descriptors. */
+	if (debug_flag == FALSE) {
+		close(STDIN_FILENO);
+		close(STDOUT_FILENO);
+		close(STDERR_FILENO);
+	}
+
 	/*
 	 *  Receive user requests
 	 */
@@ -923,7 +969,7 @@ int main(int argc, char *argv[])
 			}
 			radlog(L_ERR, "Unexpected error in select(): %s",
 					strerror(errno));
-			sig_fatal(101);
+			exit(1);
 		}
 
 		/*
@@ -1060,6 +1106,7 @@ int main(int argc, char *argv[])
  *  	 a client network access server.
  *
  *  	PW_AUTHENTICATION_ACK
+ *  	PW_ACCESS_CHALLENGE
  *  	PW_AUTHENTICATION_REJECT
  *  	PW_ACCOUNTING_RESPONSE - Reply from a remote Radius server.
  *  	 Relay reply back to original NAS.
@@ -1071,7 +1118,7 @@ int rad_process(REQUEST *request, int dospawn)
 
 	fun = NULL;
 
-	assert(request->magic == REQUEST_MAGIC);
+	rad_assert(request->magic == REQUEST_MAGIC);
 
 	switch(request->packet->code) {
 		default:
@@ -1116,6 +1163,7 @@ int rad_process(REQUEST *request, int dospawn)
 			break;
 
 		case PW_AUTHENTICATION_ACK:
+		case PW_ACCESS_CHALLENGE:
 		case PW_AUTHENTICATION_REJECT:
 		case PW_ACCOUNTING_RESPONSE:
 			/*
@@ -1159,7 +1207,7 @@ int rad_process(REQUEST *request, int dospawn)
 		return 0;
 	}
 	
-	assert(request->magic == REQUEST_MAGIC);
+	rad_assert(request->magic == REQUEST_MAGIC);
 
 	/*
 	 *  The request passes many of our sanity checks.  From
@@ -1229,6 +1277,11 @@ static void rad_reject(REQUEST *request)
 		case PW_AUTHENTICATION_REQUEST:
 			request->reply->code = PW_AUTHENTICATION_REJECT; 
 
+			/*
+			 *  Perform RFC limitations on outgoing replies.
+			 */
+			rfc_clean(request->reply);
+
 			/*
 			 *  Need to copy Proxy-State from request->packet->vps
 			 */
@@ -1242,7 +1295,7 @@ static void rad_reject(REQUEST *request)
 	 *  If a reply exists, send it.
 	 */
 	if (request->reply->code != 0) 
-		rad_send(request->reply, request->secret);
+		rad_send(request->reply, request->packet, request->secret);
 }
 
 /*
@@ -1251,24 +1304,57 @@ static void rad_reject(REQUEST *request)
 static void rfc_clean(RADIUS_PACKET *packet)
 {
 	VALUE_PAIR *vps = NULL;
-	
+
 	switch (packet->code) {
-		default:
-			break;
-			
-			/*
-			 *  Authentication REJECT's can have only
-			 *  Reply-Mesaage and Proxy-State.  We delete
-			 *  everything other than Reply-Message, and
-			 *  Proxy-State is added below, just before
-			 *  the reply is sent.
-			 */
-		case PW_AUTHENTICATION_REJECT:
-			pairmove2(&vps, &(packet->vps), PW_REPLY_MESSAGE);
-			pairfree(&packet->vps);
-			packet->vps = vps;
-			break;
+		/*
+		 *	In the default case, we just move all of the
+		 *	attributes over.
+		 */
+	default:
+		vps = packet->vps;
+		packet->vps = NULL;
+		break;
+		
+		/*
+		 *	Accounting responses can only contain
+		 *	Proxy-State and VSA's.  Note that we do NOT
+		 *	move the Proxy-State attributes over, as the
+		 *	Proxy-State attributes in this packet are NOT
+		 *	the right ones to use.  The reply function
+		 *	takes care of copying those attributes from
+		 *	the original request, which ARE the right ones
+		 *	to use.
+		 */
+	case PW_ACCOUNTING_RESPONSE:
+		pairmove2(&vps, &(packet->vps), PW_VENDOR_SPECIFIC);
+		break;
+
+		/*
+		 *	Authentication REJECT's can have only
+		 *	EAP-Message, Message-Authenticator
+		 *	Reply-Message and Proxy-State.
+		 *
+		 *	We delete everything other than these.
+		 *	Proxy-State is added below, just before the
+		 *	reply is sent.
+		 */
+	case PW_AUTHENTICATION_REJECT:
+		pairmove2(&vps, &(packet->vps), PW_EAP_MESSAGE);
+		pairmove2(&vps, &(packet->vps), PW_MESSAGE_AUTHENTICATOR);
+		pairmove2(&vps, &(packet->vps), PW_REPLY_MESSAGE);
+		pairmove2(&vps, &(packet->vps), PW_VENDOR_SPECIFIC);
+		break;
 	}
+
+	/*
+	 *	Move the newly cleaned attributes over.
+	 */
+	pairfree(&packet->vps);
+	packet->vps = vps;
+
+	/*
+	 *	FIXME: Perform other, more generic sanity checks.
+	 */
 }
 
 /* 
@@ -1334,7 +1420,7 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 		original = NULL;
 	}
 
-	assert(request->magic == REQUEST_MAGIC);
+	rad_assert(request->magic == REQUEST_MAGIC);
 	
 	/*
 	 *  Decode the packet, verifying it's signature,
@@ -1385,7 +1471,7 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 	 *  We have the semaphore, and have decoded the packet.
 	 *  Let's process the request.
 	 */
-	assert(request->magic == REQUEST_MAGIC);
+	rad_assert(request->magic == REQUEST_MAGIC);
 
 	/* 
 	 *  FIXME:  All this lowercase/nospace junk will be moved
@@ -1467,7 +1553,7 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 	 *  attributes from the request to the tail of the reply,
 	 *  and send the packet.
 	 */
-	assert(request->magic == REQUEST_MAGIC);
+	rad_assert(request->magic == REQUEST_MAGIC);
 	if (request->reply->code != 0) {
 		VALUE_PAIR *vp = NULL;
 
@@ -1483,7 +1569,7 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 		if (vp != NULL) 
 			pairadd(&(request->reply->vps), vp);
 
-		rad_send(request->reply, request->secret);
+		rad_send(request->reply, request->packet, request->secret);
 	}
 
 	/*
@@ -1670,7 +1756,7 @@ static struct timeval *rad_clean_list(time_t now)
 	last_request = NULL;
 	DEBUG2("--- Walking the entire request list ---");
 
-#if WITH_THREAD_POOL
+#if HAVE_PTHREAD_H
 	/*
 	 *  Only clean the thread pool if we've spawned child threads.
 	 */
@@ -1774,7 +1860,7 @@ static REQUEST *rad_check_list(REQUEST *request)
 						" to client %s:%d - ID: %d", client_name(curreq->packet->src_ipaddr),
 						curreq->packet->src_port, curreq->packet->id);
 
-				rad_send(curreq->reply, curreq->secret);
+				rad_send(curreq->reply, curreq->packet, curreq->secret);
 
 				/*
 				 *  There's no reply, but maybe there's
@@ -1790,7 +1876,7 @@ static REQUEST *rad_check_list(REQUEST *request)
 						       curreq->proxy->id);
 						
 						curreq->proxy_next_try = request->timestamp + proxy_retry_delay;
-						rad_send(curreq->proxy, curreq->proxysecret);
+						rad_send(curreq->proxy, curreq->packet, curreq->proxysecret);
 					} else {
 						DEBUG2("Ignoring duplicate authentication packet"
 						       " from client %s:%d - ID: %d, as the proxy reply is currently being processed.",
@@ -1885,35 +1971,13 @@ static REQUEST *rad_check_list(REQUEST *request)
 	return request;
 }
 
-#ifndef WITH_THREAD_POOL
-#if HAVE_PTHREAD_H
-typedef struct spawn_thread_t {
-	REQUEST *request;
-	RAD_REQUEST_FUNP fun;
-} spawn_thread_t;
-
-/*
- *  Spawn a new child thread to handle this request, and ONLY
- *  this request.
- */
-static void *rad_spawn_thread(void *arg)
-{
-	spawn_thread_t *data = (spawn_thread_t *)arg;
-	
-	pthread_setcancelstate(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
-
-	rad_respond(data->request, data->fun);
-	free(data);
-	return NULL;
-}
-#endif
-#endif
-
 /*
  *  If we're using the thread pool, then the function in
  *  'threads.c' replaces this one.
+ *
+ *  This code is NOT well tested, and should NOT be used!
  */
-#ifndef WITH_THREAD_POOL
+#ifndef HAVE_PTHREAD_H
 /*
  *  Spawns a child process or thread to perform
  *  authentication/accounting and respond to RADIUS clients.
@@ -1927,43 +1991,6 @@ static int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 	 * we refuse to handle SIGCHLDs normally until we're finished. */
 	signal(SIGCHLD, queue_sig_cleanup);
 
-#if HAVE_PTHREAD_H
-	{
-              int rcode;
-              spawn_thread_t *data;
-              pthread_attr_t attr;
-
-              data = (spawn_thread_t *) rad_malloc(sizeof(spawn_thread_t));
-              memset(data, 0, sizeof(data));
-              data->request = request;
-              data->fun = fun;
-
-              /*
-               *      Initialize the thread's attributes to detached.
-               *
-               *      We could call pthread_detach() later, but if
-               *      the thread exits between the create & detach
-               *      calls, it will need to be joined, which will
-               *      never happen.
-               */
-              pthread_attr_init(&attr);
-              pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-
-              /*
-               *  Create a child thread, complaining on error.
-               */
-              rcode = pthread_create(&request->child_pid, &attr,
-                                     rad_spawn_thread, data);
-              if (rcode != 0) {
-                      radlog(L_ERR, "Thread create failed for request from nas %s - ID: %d : %s",
-                             nas_name2(request->packet), request->packet->id,
-			     strerror(errno));
-                      retval = -1;
-                      goto exit_child_critsec;
-              }
-              pthread_attr_destroy(&attr);
-	}
-#else
 	/*
 	 *  fork our child
 	 */
@@ -1990,7 +2017,6 @@ static int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 	 *  Register the Child
 	 */
 	request->child_pid = child_pid;
-#endif
 
 exit_child_critsec:
 	signal(SIGCHLD, sig_cleanup);
@@ -1999,10 +2025,7 @@ exit_child_critsec:
 	}
 	return retval;
 }
-#endif /* WITH_THREAD_POOL */
 
-
-#ifndef HAVE_PTHREAD_H
 static int sig_cleanup_walker(REQUEST *req, void *data)
 {
 	int pid = (int)data;
@@ -2010,9 +2033,10 @@ static int sig_cleanup_walker(REQUEST *req, void *data)
 			return RL_WALK_CONTINUE;
 		}
 	req->child_pid = NO_SUCH_CHILD_PID;
+	req->finished = TRUE;
 	return 0;
 }
-#endif
+#endif /* HAVE_PTHREAD_H */
 
 /* used in critical section */
 void queue_sig_cleanup(int sig) {
@@ -2025,9 +2049,9 @@ void queue_sig_cleanup(int sig) {
 /*ARGSUSED*/
 void sig_cleanup(int sig)
 {
-#ifndef HAVE_PTHREAD_H
 	int status;
 	child_pid_t pid;
+#ifndef HAVE_PTHREAD_H
 	REQUEST *curreq;
 #endif
 
@@ -2043,13 +2067,11 @@ void sig_cleanup(int sig)
 	reset_signal(SIGCHLD, sig_cleanup);
 	
 	/*
-	 *  If we're using pthreads, then there are NO child processes,
-	 *  so the waitpid() call, and the following code, is useless.
+	 *	Wait for the child, without hanging.
 	 */
-#ifndef HAVE_PTHREAD_H
 	for (;;) {
 		pid = waitpid((pid_t)-1, &status, WNOHANG);
-		if (pid <= 0)
+		if ((int)pid <= 0)
 			return;
 
 		/*
@@ -2065,6 +2087,12 @@ void sig_cleanup(int sig)
 			exit(1);
 		}
 
+		/*
+		 *	If we have pthreads, then the only children
+		 *	are from Exec-Program.  We don't care about them,
+		 *	so once we've grabbed their PID's, we're done.
+		 */
+#ifndef HAVE_PTHREAD_H
 		/*
 		 *  Loop over ALL of the active requests, looking
 		 *  for the one which caused the signal.
@@ -2072,9 +2100,8 @@ void sig_cleanup(int sig)
 		if (rl_walk(sig_cleanup_walker, (void*)pid) != 0) {
 			radlog(L_ERR, "Failed to cleanup child %d", pid);
 		}
-
-	}
 #endif /* !defined HAVE_PTHREAD_H */
+	}
 }
 
 /*
@@ -2083,17 +2110,10 @@ void sig_cleanup(int sig)
 static void usage(void)
 {
 	fprintf(stderr,
-			"Usage: %s [-a acct_dir] [-d db_dir] [-l log_dir] [-i address] [-p port] [-"
-#if defined(WITH_DBM) || defined(WITH_NDBM)
-			"b"
-#endif
-			"AcfnsSvXxyz]\n", progname);
+			"Usage: %s [-a acct_dir] [-d db_dir] [-l log_dir] [-i address] [-p port] [-AcfnsSvXxyz]\n", progname);
 	fprintf(stderr, "Options:\n\n");
 	fprintf(stderr, "  -a acct_dir     use accounting directory 'acct_dir'.\n");
 	fprintf(stderr, "  -A              Log auth detail.\n");
-#if defined(WITH_DBM) || defined(WITH_NDBM)
-	fprintf(stderr, "  -b              Use DBM.\n");
-#endif
 	fprintf(stderr, "  -d db_dir       Use database directory 'db_dir'.\n");
 	fprintf(stderr, "  -f              Run as a foreground process, not a daemon.\n");
 	fprintf(stderr, "  -h              Print this help message.\n");
@@ -2118,6 +2138,7 @@ static void usage(void)
  */
 static void sig_fatal(int sig)
 {
+	int child_sig;
 	const char *me = "MASTER: ";
 
 	if (radius_pid != getpid()) {
@@ -2125,30 +2146,32 @@ static void sig_fatal(int sig)
 	}
 
 	switch(sig) {
-		case 101:
-			radlog(L_ERR, "%sfailed in select() - exit.", me);
-			break;
 		case SIGTERM:
 			radlog(L_INFO, "%sexit.", me);
+			child_sig = SIGTERM;
 			break;
 		default:
 			radlog(L_ERR, "%sexit on signal (%d)", me, sig);
+			child_sig = SIGKILL;
 			break;
 	}
 
+
+#ifndef HAVE_PTHREAD_H
 	/*
 	 *  We're running as a daemon, we're the MASTER daemon,
 	 *  and we got a fatal signal.  Tear the rest of the
 	 *  daemons down, as something absolutely horrible happened.
 	 */
 	if ((debug_flag == 0) && (dont_fork == 0) &&
-			(radius_pid == getpid())) {
+	    (radius_pid == getpid())) {
 		/*
 		 *      Kill all of the processes in the current
 		 *  process group.
 		 */
-		kill(0, SIGKILL);
+		kill(0, child_sig);
 	}
+#endif
 
 	exit(sig == SIGTERM ? 0 : 1);
 }
@@ -2210,8 +2233,20 @@ static REQUEST *proxy_check_list(REQUEST *request)
 				request_free(&request);
 				return NULL;
 			}
-		}
-
+		} else if ((oldreq->reply->code != 0) ||
+			   (oldreq->finished)) {
+			/*
+			 *	The proxy reply has arrived too late,
+			 *	the old request has timed out and is
+			 *	finished.  The client has received a
+			 *	response, so there is nothing that can
+			 *	be done. Delete the reply, and return
+			 *	NULL.
+			 */
+			radlog(L_ERR, "Proxy reply arrived too late. Try increasing retry_delay");
+			request_free(&request);
+			return NULL;
+ 		}
 	} else {
 		/*
 		 *  If we haven't found the old request, complain.
@@ -2249,7 +2284,7 @@ static int refresh_request(REQUEST *request, void *data)
 	time_t difference;
 	child_pid_t child_pid;
 
-	assert(request->magic == REQUEST_MAGIC);
+	rad_assert(request->magic == REQUEST_MAGIC);
 
 	/*
 	 *  If the request has finished processing,
@@ -2299,7 +2334,7 @@ static int refresh_request(REQUEST *request, void *data)
 				 *  This request seems to have hung
 				 *   - kill it
 				 */
-#if WITH_THREAD_POOL || HAVE_PTHREAD_H
+#if HAVE_PTHREAD_H
 				radlog(L_ERR, "Killing unresponsive thread for request %d",
 				       request->number);
 				pthread_cancel(child_pid);
@@ -2383,6 +2418,7 @@ static int refresh_request(REQUEST *request, void *data)
 	if (request->proxy_try_count == 0) {
 		request->finished = TRUE;
 		rad_reject(request);
+		realm_disable(request->proxy->dst_ipaddr);
 		goto setup_timeout;
 	}
 
@@ -2419,12 +2455,12 @@ static int refresh_request(REQUEST *request, void *data)
 	 *  If we HAVE seen it, then we SHOULD NOT be bugging the
 	 *  home server!
 	 */
-	assert(request->proxy_reply == NULL);
+	rad_assert(request->proxy_reply == NULL);
 
 	/*
 	 *  Send the proxy packet.
 	 */
-	rad_send(request->proxy, request->proxysecret);
+	rad_send(request->proxy, NULL, request->proxysecret);
 
 setup_timeout:
 	/*
@@ -2450,9 +2486,15 @@ setup_timeout:
 		 *
 		 *  Wake up when it's time to re-send
 		 *  the proxy request.
+		 *
+		 *  But in synchronous proxy, we don't retry but we update
+		 *  the next retry time as NAS has not resent the request
+		 *  in the given retry window.
 		 */
+		if (proxy_synchronous) {
+			request->proxy_next_try = info->now + proxy_retry_delay;
+		}
 		difference = request->proxy_next_try - info->now;
-		
 	} else {
 		/*
 		 *  The request is NOT finished.
