@@ -3,7 +3,7 @@
  *		Can also be installed as fingerd on the UNIX
  *		machine RADIUS runs on.
  *
- * Version:	$Id: radwho.c,v 1.27 2001/06/18 22:53:52 cmiller Exp $
+ * Version:	$Id: radwho.c,v 1.30 2001/10/30 17:45:25 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-"$Id: radwho.c,v 1.27 2001/06/18 22:53:52 cmiller Exp $";
+"$Id: radwho.c,v 1.30 2001/10/30 17:45:25 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -70,7 +70,8 @@ static int showptype = 0;
 static int showcid = 0;
 int debug_flag = 0;
 const char *progname = "radwho";
-const char *radlog_dir = "stdout";
+const char *radlog_dir = NULL;
+radlog_dest_t radlog_dest = RADLOG_FILES;
 
 int proxy_synchronous = TRUE;
 const char *radius_dir = NULL;
@@ -81,6 +82,7 @@ int acct_port;
 uint32_t myip = INADDR_ANY;
 int proxy_retry_delay = RETRY_DELAY;
 int proxy_retry_count = RETRY_COUNT;
+int proxy_dead_time;
 int log_stripped_names;
 struct main_config_t mainconfig;
 
