@@ -1,7 +1,7 @@
 /*
  * radzap.c	Zap a user from the radutmp and radwtmp file.
  *
- * Version:	$Id: radzap.c,v 1.21 2001/09/19 14:20:28 aland Exp $
+ * Version:	$Id: radzap.c,v 1.24 2001/10/30 17:45:26 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -57,6 +57,7 @@ int proxy_retry_count = RETRY_COUNT;
 int proxy_dead_time;
 int log_stripped_names;
 uint32_t myip = INADDR_ANY;
+radlog_dest_t radlog_dest = RADLOG_FILES;
 struct main_config_t mainconfig;
 
 /*
@@ -100,8 +101,8 @@ static void radutmp_unlock(int fd)
 #endif
 }
 
-static int radutmp_lookup(struct radutmp *u, uint32_t nasaddr, int port,
-		const char *user)
+static int radutmp_lookup(struct radutmp *u, uint32_t nasaddr,
+		uint32_t port, const char *user)
 {
 	int fd;
 
@@ -116,7 +117,7 @@ static int radutmp_lookup(struct radutmp *u, uint32_t nasaddr, int port,
 		 */
 		while (read(fd, u, sizeof(*u)) == sizeof(*u)) {
 			if ((nasaddr != 0 && nasaddr != u->nas_address) ||
-					(port >= 0  && port    != u->nas_port) ||
+					(port != u->nas_port) ||
 					(user != NULL &&
 					strncmp(u->login, user, sizeof u->login) != 0) ||
 					u->type != P_LOGIN)
@@ -142,7 +143,7 @@ int main(int argc, char **argv)
 	CONF_SECTION *cs;
 	NAS *nas;
 	uint32_t ip = 0;
-	int nas_port = -1;
+	uint32_t nas_port = ~0;
 	char *user = NULL;
 	char *s;
 	char buf[256];
@@ -154,7 +155,7 @@ int main(int argc, char **argv)
 		acct_port = atoi(argv[1]);
 		argc -= 2, argv+=2;
 	}
-	if (argc < 1 || argc > 3 || argv[1][0] == '-') {
+	if (argc < 1 || argc > 3 || argv[0][0] == '-') {
 		fprintf(stderr, "Usage: radzap termserver [port] [user]\n");
 		fprintf(stderr, "       radzap is only an admin tool to clean the radutmp file!\n");
 		exit(1);
@@ -162,7 +163,7 @@ int main(int argc, char **argv)
 	if (argc > 1) {
 		s = argv[1];
 		if (*s == 's' || *s == 'S') s++;
-		nas_port = atoi(s);
+		nas_port = strtoul(s, NULL, 10);
 	}
 	if (argc > 2) 
 		user = argv[2];
@@ -195,13 +196,13 @@ int main(int argc, char **argv)
 	if (nas != NULL) 
 		ip = nas->ipaddr;
 
-	printf("%s: zapping termserver %s, port %d",
+	printf("%s: zapping termserver %s, port %u",
 		progname, ip_hostname(buf, sizeof(buf), ip), nas_port);
 	if (user != NULL) 
 		printf(", user %s", user);
 	printf("\n");
 
-	if (nas_port < 0) {
+	if (nas_port == ~0) {
 		return do_accton_packet(ip);
 	}
 
