@@ -1,7 +1,7 @@
 /*
  * threads.c	request threading support
  *
- * Version:	$Id: threads.c,v 1.34 2001/08/17 19:45:27 aland Exp $
+ * Version:	$Id: threads.c,v 1.38 2001/11/07 15:20:06 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 #include "conffile.h"
 
 static const char rcsid[] =
-"$Id: threads.c,v 1.34 2001/08/17 19:45:27 aland Exp $";
+"$Id: threads.c,v 1.38 2001/11/07 15:20:06 aland Exp $";
 
 #define SEMAPHORE_LOCKED	(0)
 #define SEMAPHORE_UNLOCKED	(1)
@@ -119,7 +119,7 @@ static const CONF_PARSER thread_config[] = {
 static void *request_handler_thread(void *arg)
 {
 	THREAD_HANDLE	*self = (THREAD_HANDLE *) arg;
-#if HAVE_SIGPROCMASK
+#if HAVE_PTHREAD_SIGMASK
 	sigset_t set;
 
 	/*
@@ -134,8 +134,9 @@ static void *request_handler_thread(void *arg)
 	 */
 	sigemptyset(&set);
 	sigaddset(&set, SIGHUP);
-	sigaddset(&set, SIGCHLD);
-	sigprocmask(SIG_BLOCK, &set, NULL);
+	sigaddset(&set, SIGINT);
+	sigaddset(&set, SIGQUIT);
+	pthread_sigmask(SIG_BLOCK, &set, NULL);
 #endif
 	
 	/*
@@ -147,7 +148,9 @@ static void *request_handler_thread(void *arg)
 		 */
 		DEBUG2("Thread %d waiting to be assigned a request",
 				self->thread_num);
-		sem_wait(&self->semaphore);
+		if (sem_wait(&self->semaphore) < 0) {
+			break;
+		}
 
 		/*
 		 *	If we've been told to kill ourselves,
@@ -241,6 +244,13 @@ static void move2tail(THREAD_HANDLE *handle)
 		return;
 	}
 
+	/*
+	 *	It's already at the tail.  Stop.
+	 */
+	if (thread_pool.tail == handle) {
+		return;
+	}
+
 	rad_assert(thread_pool.total_threads >= 1);
 	prev = handle->prev;
 	next = handle->next;
@@ -615,7 +625,27 @@ int thread_pool_clean(time_t now)
 				handle->status = THREAD_CANCELLED;
 				sem_post(&handle->semaphore);
 				spare--;
-				return 0;
+				break;
+			}
+		}
+	}
+  
+	/*
+	 *	If the thread has handled too many requests, then make it
+	 *	exit.
+	 */
+	if (thread_pool.max_requests_per_thread > 0) {
+		for (handle = thread_pool.head; handle; handle = next) {
+			next = handle->next;
+
+			/*
+			 *	Not handling a request, we can check it.
+			 */
+			if ((handle->request == NULL) &&
+			    (handle->request_count > thread_pool.max_requests_per_thread)) {
+				delete_thread(handle);
+				handle->status = THREAD_CANCELLED;
+				sem_post(&handle->semaphore);
 			}
 		}
 	}
