@@ -1,7 +1,7 @@
 /*
  * util.c	Various utility functions.
  *
- * Version:     $Id: util.c,v 1.27 2001/03/05 03:42:11 cmiller Exp $
+ * Version:     $Id: util.c,v 1.28 2001/08/17 19:45:27 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: util.c,v 1.27 2001/03/05 03:42:11 cmiller Exp $";
+static const char rcsid[] = "$Id: util.c,v 1.28 2001/08/17 19:45:27 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -203,3 +203,14 @@ void xfree(const char *ptr)
 {
 	free((char *)ptr);
 }
+
+/*
+ *	Logs an error message and aborts the program
+ *
+ */
+
+void rad_assert_fail (const char *file, unsigned int line)
+{
+	radlog(L_ERR|L_CONS, "Assertion failed in %s, line %u", file, line);
+	abort();
+}
