@@ -3,7 +3,7 @@
  *                      before sending reply to the NAS/Server that sent
  *                      it to us.
  *
- * Version:      $Id: rlm_attr_filter.c,v 1.5 2001/08/27 17:52:23 aland Exp $
+ * Version:      $Id: rlm_attr_filter.c,v 1.6 2001/11/09 00:53:09 kkalev Exp $
  *
  *   This program is is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License, version 2 if the
@@ -41,7 +41,7 @@
 #include	"radiusd.h"
 #include	"modules.h"
 
-static const char rcsid[] = "$Id: rlm_attr_filter.c,v 1.5 2001/08/27 17:52:23 aland Exp $";
+static const char rcsid[] = "$Id: rlm_attr_filter.c,v 1.6 2001/11/09 00:53:09 kkalev Exp $";
 
 struct attr_filter_instance {
 
@@ -322,7 +322,7 @@ static int attr_filter_authorize(void *instance, REQUEST *request)
 
 		    if(reply_item != (VALUE_PAIR *)NULL) {
 
-		        compare = simplepaircmp(reply_item, check_item); 
+			compare = simplepaircmp(request, reply_item, check_item);
 
 			/* DEBUG2("    attr_filter: compare = %d", compare); */
 
