@@ -1,7 +1,7 @@
 /*
  * rlm_detail.c	accounting:    Write the "detail" files.
  *
- * Version:	$Id: rlm_detail.c,v 1.24 2001/03/14 20:08:43 aland Exp $
+ * Version:	$Id: rlm_detail.c,v 1.25 2001/11/07 15:55:14 aland Exp $
  *
  *  This program is is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2 if the
@@ -19,7 +19,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: rlm_detail.c,v 1.24 2001/03/14 20:08:43 aland Exp $";
+static const char rcsid[] = "$Id: rlm_detail.c,v 1.25 2001/11/07 15:55:14 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -184,9 +184,9 @@ static int detail_accounting(void *instance, REQUEST *request)
 		 *	Add non-protocol attibutes.
 		 */
 		fprintf(outfp, "\tTimestamp = %ld\n", request->timestamp);
-		if (request->packet->verified)
+		if (request->packet->verified == 2)
 			fputs("\tRequest-Authenticator = Verified\n", outfp);
-		else
+		else if (request->packet->verified == 1)
 			fputs("\tRequest-Authenticator = None\n", outfp);
 		fputs("\n", outfp);
 		fclose(outfp);
