@@ -29,8 +29,18 @@
  *	  object, which contains default values for RADIUS users
  *	- Added "profile_attribute" directive, which specifies user object 
  *	  attribute pointing to radiusProfile object.
+ * Nov 2001, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Added support for adding the user password to the check. Based on
+ *	  the password_header directive rlm_ldap will strip the
+ *	  password header if needed. This will make support for CHAP much easier.
+ *	- Added module messages when we reject a user.
+ *	- Added ldap_groupcmp to allow searching for user group membership.
+ *	- Added ldap_xlat to allow ldap urls in xlat strings. Something like:
+ *	  %{ldap:ldap:///dc=company,dc=com?cn?sub?uid=user}
+ * Nov 2001, Gordon Tetlow <gordont@gnf.org>
+ *	- Do an xlat on the access_group attribute.
  */
-static const char rcsid[] = "$Id: rlm_ldap.c,v 1.50 2001/08/28 20:01:48 aland Exp $";
+static const char rcsid[] = "$Id: rlm_ldap.c,v 1.59 2001/11/27 22:44:25 kkalev Exp $";
 
 #include "autoconf.h"
 
@@ -81,6 +91,7 @@ typedef struct {
 	struct timeval  timeout;
 	int             debug;
 	int             tls_mode;
+	int		start_tls;
 	char           *login;
 	char           *password;
 	char           *filter;
@@ -89,7 +100,11 @@ typedef struct {
 	char           *profile_attr;
 	char           *access_group;
 	char           *access_attr;
+	char           *passwd_hdr;
+	char           *passwd_attr;
 	char           *dictionary_mapping;
+	char	       *groupname_attr;
+	char	       *groupmemb_filt;
 	TLDAP_RADIUS   *check_item_map;
 	TLDAP_RADIUS   *reply_item_map;
 	LDAP           *ld;
@@ -107,15 +122,20 @@ static CONF_PARSER module_config[] = {
 	/* allow server unlimited time for search (server-side limit) */
 	{"timelimit", PW_TYPE_INTEGER, offsetof(ldap_instance,timelimit), NULL, "20"},
 	{"identity", PW_TYPE_STRING_PTR, offsetof(ldap_instance,login), NULL, ""},
+	{"start_tls", PW_TYPE_BOOLEAN, offsetof(ldap_instance,start_tls), NULL, "no"},
 	{"password", PW_TYPE_STRING_PTR, offsetof(ldap_instance,password), NULL, ""},
 	{"basedn", PW_TYPE_STRING_PTR, offsetof(ldap_instance,basedn), NULL, NULL},
 	{"filter", PW_TYPE_STRING_PTR, offsetof(ldap_instance,filter), NULL, "(uid=%u)"},
 	{"default_profile", PW_TYPE_STRING_PTR, offsetof(ldap_instance,default_profile), NULL, NULL},
 	{"profile_attribute", PW_TYPE_STRING_PTR, offsetof(ldap_instance,profile_attr), NULL, NULL},
 	{"access_group", PW_TYPE_STRING_PTR, offsetof(ldap_instance,access_group), NULL, NULL},
+	{"password_header", PW_TYPE_STRING_PTR, offsetof(ldap_instance,passwd_hdr), NULL, NULL},
+	{"password_attribute", PW_TYPE_STRING_PTR, offsetof(ldap_instance,passwd_attr), NULL, NULL},
 	/* LDAP attribute name that controls remote access */
 	{"access_attr", PW_TYPE_STRING_PTR, offsetof(ldap_instance,access_attr), NULL, NULL},
 	/* file with mapping between LDAP and RADIUS attributes */
+	{"groupname_attribute", PW_TYPE_STRING_PTR, offsetof(ldap_instance,groupname_attr), NULL, "cn"},
+	{"groupmembership_filter", PW_TYPE_STRING_PTR, offsetof(ldap_instance,groupmemb_filt), NULL, "(|(&(objectClass=GroupOfNames)(member=%{Ldap-UserDn}))(&(objectClass=GroupOfUniqueNames)(uniquemember=%{Ldap-UserDn})))"},
 	{"dictionary_mapping", PW_TYPE_STRING_PTR, offsetof(ldap_instance,dictionary_mapping), NULL, "${confdir}/ldap.attrmap"},
 	{"ldap_debug", PW_TYPE_INTEGER, offsetof(ldap_instance,ldap_debug), NULL, "0x0000"},
 
@@ -130,6 +150,8 @@ static CONF_PARSER module_config[] = {
 static void     fieldcpy(char *, char **);
 #endif
 static VALUE_PAIR *ldap_pairget(LDAP *, LDAPMessage *, TLDAP_RADIUS *,VALUE_PAIR **);
+static int ldap_groupcmp(void *, REQUEST *, VALUE_PAIR *, VALUE_PAIR *, VALUE_PAIR *, VALUE_PAIR **);
+static int ldap_xlat(void *,REQUEST *, char *, char *,int, RADIUS_ESCAPE_STRING);
 static LDAP    *ldap_connect(void *instance, const char *, const char *, int, int *);
 static int     read_mappings(ldap_instance* inst);
 
@@ -155,6 +177,7 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 
 	if (inst->server == NULL) {
 		radlog(L_ERR, "rlm_ldap: missing 'server' directive.");
+		free(inst);
 		return -1;
 	}
  
@@ -166,6 +189,12 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 	inst->check_item_map = NULL;
 	inst->ld = NULL;
 
+	paircompare_register(PW_GROUP, PW_USER_NAME, ldap_groupcmp, inst);
+#ifdef PW_GROUP_NAME /* compat */
+	paircompare_register(PW_GROUP_NAME, PW_USER_NAME, ldap_groupcmp, inst);
+#endif
+	xlat_register("ldap",ldap_xlat,inst);
+
 	if (read_mappings(inst) != 0) {
 		radlog(L_ERR, "rlm_ldap: Reading dictionary mappings from file %s failed",
 		       inst->dictionary_mapping);
@@ -177,6 +206,7 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 	return 0;
 }
 
+
 /*
  * read_mappings(...) reads a ldap<->radius mappings file to inst->reply_item_map and inst->check_item_map
  */
@@ -243,6 +273,9 @@ read_mappings(ldap_instance* inst)
 
 		if ( (pair->attr == NULL) || (pair->radius_attr == NULL) ) {
 			radlog(L_ERR, "rlm_ldap: Out of memory");
+			if (pair->attr) free(pair->attr);
+			if (pair->radius_attr) free(pair->radius_attr);
+			free(pair);
 			fclose(mapfile);
 			return -1;
 		}
@@ -313,6 +346,210 @@ perform_search(void *instance, char *search_basedn, int scope, char *filter, cha
 	return res;
 }
 
+
+/*
+ * ldap_groupcmp(). Implement the Group == "group" filter
+ */
+
+static int ldap_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check,
+                VALUE_PAIR *check_pairs, VALUE_PAIR **reply_pairs)
+{
+        char            filter[MAX_AUTH_QUERY_LEN];
+        char            *group_dn;
+        int             res;
+        LDAPMessage     *result = NULL;
+        LDAPMessage     *msg = NULL;
+        char            basedn[1024];
+        ldap_instance   *inst = instance;
+
+        check_pairs = check_pairs;
+        reply_pairs = reply_pairs;
+
+	DEBUG("rlm_ldap: Entering ldap_groupcmp()");
+
+        if (check->strvalue == NULL || check->length == 0){
+                DEBUG("rlm_ldap::ldap_groupcmp: Illegal group name");
+                return 1;
+        }
+
+        if (req == NULL){
+                DEBUG("rlm_ldap::ldap_groupcmp: NULL request");
+                return 1;
+        }
+
+        if (!radius_xlat(basedn, sizeof(basedn), inst->basedn, req, NULL)) {
+                DEBUG("rlm_ldap::ldap_groupcmp: unable to create basedn.");
+                return 1;
+        }
+
+        if ((pairfind(req->packet->vps, LDAP_USERDN)) == NULL){
+                char            *user_dn = NULL;
+
+                if (!radius_xlat(filter, MAX_AUTH_QUERY_LEN, inst->filter, req, NULL)) {
+                        DEBUG("rlm_ldap::ldap_groupcmp: unable to create filter");
+                        return 1;
+                }
+                if ((res = perform_search(inst, basedn, LDAP_SCOPE_SUBTREE, filter, NULL, &result)) != RLM_MODULE_OK) {
+                        DEBUG("rlm_ldap::ldap_groupcmp: search failed");
+                        return 1;
+                }
+                if ((msg = ldap_first_entry(inst->ld, result)) == NULL) {
+                        DEBUG("rlm_ldap::ldap_groupcmp: ldap_first_entry() failed");
+                        ldap_msgfree(result);
+                        return 1;
+                }
+                if ((user_dn = ldap_get_dn(inst->ld, msg)) == NULL) {
+                        DEBUG("rlm_ldap:ldap_groupcmp:: ldap_get_dn() failed");
+                        ldap_msgfree(result);
+                        return 1;
+                }
+                /*
+                * Adding new attribute containing DN for LDAP object associated with
+                * given username
+                */
+                pairadd(&req->packet->vps, pairmake("Ldap-UserDn", user_dn, T_OP_EQ));
+                ldap_memfree(user_dn);
+                ldap_msgfree(result);
+        }
+
+        snprintf(filter,MAX_AUTH_QUERY_LEN - 1, "(%s=%s)",inst->groupname_attr,(char *)check->strvalue);
+
+        if ((res = perform_search(inst, basedn, LDAP_SCOPE_SUBTREE, filter, NULL, &result)) != RLM_MODULE_OK){
+                if (res == RLM_MODULE_NOTFOUND){
+                        DEBUG("rlm_ldap::ldap_groupcmp: Group %s not found", (char *)check->strvalue);
+                        return 1;
+                }
+                DEBUG("rlm_ldap::ldap_groupcmp: Search returned error");
+                return 1;
+        }
+        if ((msg = ldap_first_entry(inst->ld, result)) == NULL){
+                DEBUG("rlm_ldap::ldap_groupcmp: ldap_first_entry() failed");
+                ldap_msgfree(result);
+                return 1;
+        }
+        if ((group_dn = ldap_get_dn(inst->ld, msg)) == NULL){
+                DEBUG("rlm_ldap:ldap_groupcmp:: ldap_get_dn() failed");
+                ldap_msgfree(result);
+                return 1;
+        }
+	ldap_msgfree(result);
+
+
+        if(!radius_xlat(filter, MAX_AUTH_QUERY_LEN, inst->groupmemb_filt, req, NULL)){
+                DEBUG("rlm_ldap::ldap_groupcmp: unable to create filter.");
+		ldap_memfree(group_dn);
+                return 1;
+        }
+
+        if ((res = perform_search(inst, group_dn, LDAP_SCOPE_BASE, filter, NULL, &result)) != RLM_MODULE_OK){
+                if (res == RLM_MODULE_NOTFOUND){
+			DEBUG("rlm_ldap::ldap_groupcmp: User not found in group %s",group_dn);
+			ldap_memfree(group_dn);
+			return -1;
+                }
+                DEBUG("rlm_ldap::ldap_groupcmp: Search returned error");
+		ldap_memfree(group_dn);
+                return 1;
+        }
+        else{
+                DEBUG("rlm_ldap::ldap_groupcmp: User found in group %s",group_dn);
+		ldap_memfree(group_dn);
+                ldap_msgfree(result);
+        }
+
+        return 0;
+
+}
+
+/*
+ * ldap_xlat()
+ * Do an xlat on an LDAP URL
+ */
+
+static int ldap_xlat(void *instance, REQUEST *request, char *fmt, char *out, int freespace,
+				RADIUS_ESCAPE_STRING func)
+{
+	char url[MAX_STRING_LEN];
+	int res;
+	int ret = 0;
+	ldap_instance *inst = instance;
+	LDAPURLDesc *ldap_url;
+	LDAPMessage *result = NULL;
+	LDAPMessage *msg = NULL;
+	char **vals;
+
+	DEBUG("rlm_ldap: - ldap_xlat");
+	if (!radius_xlat(url, sizeof(url), fmt, request, func)) {
+		radlog (L_ERR, "rlm_ldap: Unable to create LDAP URL.\n");
+		return 0;
+	}
+	if (!ldap_is_ldap_url(url)){
+		radlog (L_ERR, "rlm_ldap: String passed does not look like an LDAP URL.\n");
+		return 0;
+	}
+	if (ldap_url_parse(url,&ldap_url)){
+		radlog (L_ERR, "rlm_ldap: LDAP URL parse failed.\n");
+		return 0;
+	}
+	if (ldap_url->lud_attrs == NULL || ldap_url->lud_attrs[0] == NULL || \
+		( ldap_url->lud_attrs[1] != NULL || ( ! strlen(ldap_url->lud_attrs[0]) || \
+		! strcmp(ldap_url->lud_attrs[0],"*") ) ) ){
+		radlog (L_ERR, "rlm_ldap: Invalid Attribute(s) request.\n");
+		ldap_free_urldesc(ldap_url);
+		return 0;
+	}
+	if (ldap_url->lud_host){
+		if (strncmp(inst->server,ldap_url->lud_host,strlen(inst->server)) != 0 || \
+				ldap_url->lud_port != inst->port){
+			DEBUG("rlm_ldap: Requested server/port is not known to this module instance.");
+			ldap_free_urldesc(ldap_url);
+			return 0;
+		}
+	}
+	if ((res = perform_search(inst, ldap_url->lud_dn, ldap_url->lud_scope, ldap_url->lud_filter, ldap_url->lud_attrs, &result)) != RLM_MODULE_OK){
+		if (res == RLM_MODULE_NOTFOUND){
+			DEBUG("rlm_ldap: Search returned not found");
+			ldap_free_urldesc(ldap_url);
+			return 0;
+		}
+		DEBUG("rlm_ldap: Search returned error");
+		ldap_free_urldesc(ldap_url);
+		return 0;
+	}
+	if ((msg = ldap_first_entry(inst->ld, result)) == NULL){
+		DEBUG("rlm_ldap: ldap_first_entry() failed");
+		ldap_msgfree(result);
+		ldap_free_urldesc(ldap_url);
+		return 0;
+	}
+	if ((vals = ldap_get_values(inst->ld, msg, ldap_url->lud_attrs[0])) != NULL) {
+		if (ldap_count_values(vals)){
+			ret = strlen(vals[0]);
+			if (ret > freespace){
+				DEBUG("rlm_ldap: Insufficient string space");
+				ldap_free_urldesc(ldap_url);
+				ldap_value_free(vals);
+				ldap_msgfree(result);
+				return 0;
+			}
+			DEBUG("rlm_ldap: Adding attribute %s, value: %s",ldap_url->lud_attrs[0],vals[0]);
+			strncpy(out,vals[0],ret);
+			ldap_free_urldesc(ldap_url);
+			ldap_value_free(vals);
+		}
+	}
+	else{
+		ldap_free_urldesc(ldap_url);
+		ldap_msgfree(result);
+		ret = 0;
+	}
+
+	DEBUG("rlm_ldap: - ldap_xlat end");
+
+	return ret;
+}
+
+
 /******************************************************************************
  *
  *      Function: rlm_ldap_authorize
@@ -332,18 +569,21 @@ ldap_authorize(void *instance, REQUEST * request)
 	LDAPMessage	*def_attr_result = NULL;
 	ldap_instance	*inst = instance;
 	char		*user_dn = NULL;
-	char		*attrs[] = {"*", NULL};
-	static char	filter[MAX_AUTH_QUERY_LEN];
-	static char 	filt_patt[] = "(| (& (objectClass=GroupOfNames) (member=%{Ldap-UserDn})) (& (objectClass=GroupOfUniqueNames) (uniquemember=%{Ldap-UserDn})))"; 
+	const char	*attrs[] = {"*", NULL};
+	char		filter[MAX_AUTH_QUERY_LEN];
+	char		basedn[1024];
 	VALUE_PAIR	*check_tmp;
 	VALUE_PAIR	*reply_tmp;
 	int		res;
 	VALUE_PAIR	**check_pairs, **reply_pairs;
 	char		**vals;
+	VALUE_PAIR      *module_msg_vp;
+	char            module_msg[MAX_STRING_LEN];
+
 
 	DEBUG("rlm_ldap: - authorize");
 
-	if(!request->username){
+	if (!request->username){
 		radlog(L_AUTH, "rlm_ldap: Attribute \"User-Name\" is required for authentication.\n");
 		return RLM_MODULE_INVALID;
 	}
@@ -351,7 +591,6 @@ ldap_authorize(void *instance, REQUEST * request)
 	check_pairs = &request->config_items;
 	reply_pairs = &request->reply->vps;
 
-
 	/*
 	 * Check for valid input, zero length names not permitted
 	 */
@@ -362,12 +601,25 @@ ldap_authorize(void *instance, REQUEST * request)
 	DEBUG("rlm_ldap: performing user authorization for %s",
 	       request->username->strvalue);
 
-	if(!radius_xlat(filter, MAX_AUTH_QUERY_LEN, inst->filter,
-			request, NULL)) 
-		radlog (L_ERR, "rlm_ldap: unable to create filter.\n"); 
+	if (!radius_xlat(filter, sizeof(filter), inst->filter,
+			 request, NULL)) {
+		radlog (L_ERR, "rlm_ldap: unable to create filter.\n");
+		return RLM_MODULE_INVALID;
+	}
 
-	if ((res = perform_search(instance, inst->basedn, LDAP_SCOPE_SUBTREE, filter, attrs, &result)) != RLM_MODULE_OK) {
+	if (!radius_xlat(basedn, sizeof(basedn), inst->basedn,
+			 request, NULL)) {
+		radlog (L_ERR, "rlm_ldap: unable to create basedn.\n");
+		return RLM_MODULE_INVALID;
+	}
+
+	if ((res = perform_search(instance, basedn, LDAP_SCOPE_SUBTREE, filter, attrs, &result)) != RLM_MODULE_OK) {
 		DEBUG("rlm_ldap: search failed");
+		if (res == RLM_MODULE_NOTFOUND){
+			snprintf(module_msg,MAX_STRING_LEN-1,"rlm_ldap: User not found");
+			module_msg_vp = pairmake("Module-Message", module_msg, T_OP_EQ);
+			pairadd(&request->packet->vps, module_msg_vp);
+		}
 		return (res);
 	}
 	if ((msg = ldap_first_entry(inst->ld, result)) == NULL) {
@@ -394,6 +646,9 @@ ldap_authorize(void *instance, REQUEST * request)
 			DEBUG("rlm_ldap: checking if remote access for %s is allowed by %s", request->username->strvalue, inst->access_attr);
 			if (!strncmp(vals[0], "FALSE", 5)) {
 				DEBUG("rlm_ldap: dialup access disabled");
+				snprintf(module_msg,MAX_STRING_LEN-1,"rlm_ldap: Access Attribute denies access");
+				module_msg_vp = pairmake("Module-Message", module_msg, T_OP_EQ);
+				pairadd(&request->packet->vps, module_msg_vp);
 				ldap_msgfree(result);
 				ldap_value_free(vals);
 				return RLM_MODULE_USERLOCK;
@@ -401,26 +656,39 @@ ldap_authorize(void *instance, REQUEST * request)
 			ldap_value_free(vals);
 		} else {
 			DEBUG("rlm_ldap: no %s attribute - access denied by default", inst->access_attr);
+			snprintf(module_msg,MAX_STRING_LEN-1,"rlm_ldap: Access Attribute denies access");
+			module_msg_vp = pairmake("Module-Message", module_msg, T_OP_EQ);
+			pairadd(&request->packet->vps, module_msg_vp);
 			ldap_msgfree(result);
 			return RLM_MODULE_USERLOCK;
 		}
 	}
 	/* Remote access controled by group membership of the user object */
 	if (inst->access_group != NULL) {
+		static char	group[MAX_AUTH_QUERY_LEN];
+
 		DEBUG("rlm_ldap: checking user membership in dialup-enabling group %s", inst->access_group);
 		/*
 		 * uniquemember appears in Netscape Directory Server's groups
 		 * since we have objectclass groupOfNames and
 		 * groupOfUniqueNames
 		 */
-		if(!radius_xlat(filter, MAX_AUTH_QUERY_LEN, filt_patt,
+		if(!radius_xlat(group, MAX_AUTH_QUERY_LEN, inst->access_group,
+				request, NULL)) 
+			radlog (L_ERR, "rlm_ldap: unable to munge group.\n"); 
+
+		if(!radius_xlat(filter, MAX_AUTH_QUERY_LEN, inst->groupmemb_filt,
 			        request, NULL)) 
 			radlog (L_ERR, "rlm_ldap: unable to create filter.\n"); 
 
-		if ((res = perform_search(instance, inst->access_group, LDAP_SCOPE_BASE, filter, NULL, &gr_result)) != RLM_MODULE_OK) {
+		if ((res = perform_search(instance, group, LDAP_SCOPE_BASE, filter, NULL, &gr_result)) != RLM_MODULE_OK) {
 			ldap_msgfree(result);
-			if (res == RLM_MODULE_NOTFOUND)
+			if (res == RLM_MODULE_NOTFOUND){
+				snprintf(module_msg,MAX_STRING_LEN-1,"rlm_ldap: User is not an access group member");
+				module_msg_vp = pairmake("Module-Message", module_msg, T_OP_EQ);
+				pairadd(&request->packet->vps, module_msg_vp);
 				return (RLM_MODULE_USERLOCK);
+			}
 			else
 				return (res);
 		} else 
@@ -480,6 +748,44 @@ ldap_authorize(void *instance, REQUEST * request)
 			ldap_value_free(vals);
 		}
 	}
+	if (inst->passwd_attr && strlen(inst->passwd_attr)){
+		VALUE_PAIR *passwd_item;
+
+		if ((passwd_item = pairfind(request->config_items, PW_PASSWORD)) == NULL){
+			char **passwd_vals;
+			char *passwd_val = NULL;
+			int passwd_len;
+
+			if ((passwd_vals = ldap_get_values(inst->ld,msg,inst->passwd_attr)) != NULL){
+				if (ldap_count_values(passwd_vals) && strlen(passwd_vals[0])){
+					passwd_val = passwd_vals[0];
+
+					if (inst->passwd_hdr && strlen(inst->passwd_hdr)){
+						passwd_val = strstr(passwd_val,inst->passwd_hdr);
+						if (passwd_val != NULL)
+							passwd_val += strlen(inst->passwd_hdr);
+						else
+							DEBUG("rlm_ldap: Password header not found in password %s for user %s", passwd_vals[0],request->username->strvalue);
+					}
+					if (passwd_val){
+						if ((passwd_item = paircreate(PW_PASSWORD,PW_TYPE_STRING)) == NULL){
+							radlog(L_ERR|L_CONS, "no memory");
+							ldap_value_free(passwd_vals);
+							ldap_msgfree(result);
+							return RLM_MODULE_FAIL;
+						}
+						passwd_len = strlen(passwd_val);
+						strncpy(passwd_item->strvalue,passwd_val,MAX_STRING_LEN - 1);
+						passwd_item->length = (passwd_len > (MAX_STRING_LEN - 1)) ? (MAX_STRING_LEN - 1) : passwd_len;
+						pairadd(&request->config_items,passwd_item);
+						DEBUG("rlm_ldap: Added password %s in check items",passwd_item->strvalue);
+					}
+				}
+				ldap_value_free(passwd_vals);
+			}
+		}
+	}
+
 
 
 	DEBUG("rlm_ldap: looking for check items in directory...");
@@ -493,7 +799,7 @@ ldap_authorize(void *instance, REQUEST * request)
 	 * specified
 	 */
 	if (pairfind(*check_pairs, PW_AUTHTYPE) == NULL)
-		pairadd(check_pairs, pairmake("Auth-Type", "LDAP", T_OP_CMP_EQ));
+		pairadd(check_pairs, pairmake("Auth-Type", "LDAP", T_OP_EQ));
 
 
 	DEBUG("rlm_ldap: looking for reply items in directory...");
@@ -523,9 +829,12 @@ ldap_authenticate(void *instance, REQUEST * request)
 	LDAPMessage    *result, *msg;
 	ldap_instance  *inst = instance;
 	char           *user_dn, *attrs[] = {"uid", NULL};
-	static char	filter[MAX_AUTH_QUERY_LEN];
+        char		filter[MAX_AUTH_QUERY_LEN];
+	char		basedn[1024];
 	int             res;
 	VALUE_PAIR     *vp_user_dn;
+	VALUE_PAIR      *module_msg_vp;
+	char            module_msg[MAX_STRING_LEN];
 
 	DEBUG("rlm_ldap: - authenticate");
 	/*
@@ -533,7 +842,7 @@ ldap_authenticate(void *instance, REQUEST * request)
 	 * anything else.
 	 */
 
-	if(!request->username){
+	if (!request->username) {
 		radlog(L_AUTH, "rlm_ldap: Attribute \"User-Name\" is required for authentication.\n");
 		return RLM_MODULE_INVALID;
 	}
@@ -552,14 +861,29 @@ ldap_authenticate(void *instance, REQUEST * request)
 		radlog(L_ERR, "rlm_ldap: empty password supplied");
 		return RLM_MODULE_INVALID;
 	}
+
 	DEBUG("rlm_ldap: login attempt by \"%s\" with password \"%s\"", 
 	       request->username->strvalue, request->password->strvalue);
-	if(!radius_xlat(filter, MAX_AUTH_QUERY_LEN, inst->filter,
-			request, NULL)) 
+
+	if (!radius_xlat(filter, sizeof(filter), inst->filter,
+			request, NULL)) {
 		radlog (L_ERR, "rlm_ldap: unable to create filter.\n"); 
+		return RLM_MODULE_INVALID;
+	}
+
+	if (!radius_xlat(basedn, sizeof(basedn), inst->basedn,
+			 request, NULL)) {
+		radlog (L_ERR, "rlm_ldap: unable to create basedn.\n");
+		return RLM_MODULE_INVALID;
+	}
 
 	while((vp_user_dn = pairfind(request->packet->vps, LDAP_USERDN)) == NULL) {
-		if ((res = perform_search(instance, inst->basedn, LDAP_SCOPE_SUBTREE, filter, attrs, &result)) != RLM_MODULE_OK) {
+		if ((res = perform_search(instance, basedn, LDAP_SCOPE_SUBTREE, filter, attrs, &result)) != RLM_MODULE_OK) {
+			if (res == RLM_MODULE_NOTFOUND){
+				snprintf(module_msg,MAX_STRING_LEN-1,"rlm_ldap: User not found");
+				module_msg_vp = pairmake("Module-Message", module_msg, T_OP_EQ);
+				pairadd(&request->packet->vps, module_msg_vp);
+			}
 			return (res);
 		}
 		if ((msg = ldap_first_entry(inst->ld, result)) == NULL) {
@@ -582,8 +906,12 @@ ldap_authenticate(void *instance, REQUEST * request)
 
 	ld_user = ldap_connect(instance, user_dn, request->password->strvalue,
 			       1, &res);
-	if (ld_user == NULL)
+	if (ld_user == NULL){
+		snprintf(module_msg,MAX_STRING_LEN-1,"rlm_ldap: Bind as user failed");
+		module_msg_vp = pairmake("Module-Message", module_msg, T_OP_EQ);
+		pairadd(&request->packet->vps, module_msg_vp);
 		return (res);
+	}
 
 	DEBUG("rlm_ldap: user %s authenticated succesfully",
 	      request->username->strvalue);
@@ -596,7 +924,7 @@ ldap_connect(void *instance, const char *dn, const char *password, int auth, int
 {
 	ldap_instance  *inst = instance;
 	LDAP           *ld;
-	int             msgid, rc;
+	int             msgid, rc, version;
 	int		ldap_errno = 0;
 	LDAPMessage    *res;
 
@@ -621,10 +949,28 @@ ldap_connect(void *instance, const char *dn, const char *password, int auth, int
 	}
 #endif
 
+#ifdef HAVE_LDAP_START_TLS
+	if (inst->start_tls) {
+		DEBUG("rlm_ldap: try to start TLS");
+		version = LDAP_VERSION3;
+		if (ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &version) == LDAP_SUCCESS) {
+			rc = ldap_start_tls_s(ld, NULL, NULL);
+			if (rc != LDAP_SUCCESS) {
+				DEBUG("rlm_ldap: ldap_start_tls_s()");
+				ldap_get_option(ld, LDAP_OPT_ERROR_NUMBER, &ldap_errno);
+				radlog(L_ERR, "rlm_ldap: could not start TLS %s", ldap_err2string(ldap_errno));
+				*result = RLM_MODULE_FAIL;
+				ldap_unbind_s(ld);
+				return (NULL);
+			}
+		}
+	}
+#endif /* HAVE_LDAP_START_TLS */
+
 	DEBUG("rlm_ldap: bind as %s/%s", dn, password);
-	msgid = ldap_bind(ld, dn, password, LDAP_AUTH_SIMPLE);
+	msgid = ldap_simple_bind(ld, dn, password);
 	if (msgid == -1) {
-		DEBUG("rlm_ldap: ldap_bind()");
+		DEBUG("rlm_ldap: ldap_simple_bind()");
 		ldap_get_option(ld, LDAP_OPT_ERROR_NUMBER, &ldap_errno);
 		radlog(L_ERR, "rlm_ldap: %s bind failed: %s", dn, ldap_err2string(ldap_errno));
 		*result = RLM_MODULE_FAIL;
@@ -695,6 +1041,14 @@ ldap_detach(void *instance)
 		free(inst->dictionary_mapping);
 	if (inst->filter)
 		free((char *) inst->filter);
+	if (inst->passwd_hdr)
+		free((char *) inst->passwd_hdr);
+	if (inst->passwd_attr)
+		free((char *) inst->passwd_attr);
+	if (inst->groupname_attr)
+		free((char *) inst->groupname_attr);
+	if (inst->groupmemb_filt)
+		free((char *) inst->groupmemb_filt);
 	if (inst->ld)
 		ldap_unbind_s(inst->ld);
 
@@ -718,6 +1072,12 @@ ldap_detach(void *instance)
 		pair = nextpair;
 	}
 
+	paircompare_unregister(PW_GROUP, ldap_groupcmp);
+#ifdef PW_GROUP_NAME
+	paircompare_unregister(PW_GROUP_NAME, ldap_groupcmp);
+#endif
+	xlat_unregister("ldap",ldap_xlat);
+
 	free(inst);
 
 	return 0;
