@@ -2,7 +2,7 @@
  * rlm_sql.c		SQL Module
  * 		Main SQL module file. Most ICRADIUS code is located in sql.c
  *
- * Version:	$Id: rlm_sql.c,v 1.67 2001/09/12 22:42:12 cmiller Exp $
+ * Version:	$Id: rlm_sql.c,v 1.68 2001/11/09 00:53:11 kkalev Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-	"$Id: rlm_sql.c,v 1.67 2001/09/12 22:42:12 cmiller Exp $";
+	"$Id: rlm_sql.c,v 1.68 2001/11/09 00:53:11 kkalev Exp $";
 
 #include "autoconf.h"
 
@@ -276,7 +276,7 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 #if 0 /* FIXME: this is the *real* authorizing */
 	vp_printlist(stderr, check_tmp);
 #endif
-	if (paircmp(request->packet->vps, check_tmp, &reply_tmp) != 0) {
+	if (paircmp(request, request->packet->vps, check_tmp, &reply_tmp) != 0) {
 		radlog(L_INFO, "rlm_sql: Pairs do not match [%s]", sqlusername);
 		return RLM_MODULE_NOTFOUND;
 	}
