@@ -2,7 +2,7 @@
  *  sql.c		rlm_sql - FreeRADIUS SQL Module
  *		Main code directly taken from ICRADIUS
  *
- * Version:	$Id: sql.c,v 1.43 2001/09/12 22:42:13 cmiller Exp $
+ * Version:	$Id: sql.c,v 1.45 2001/12/05 18:27:24 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -244,6 +244,8 @@ int sql_userparse(VALUE_PAIR ** first_pair, SQL_ROW row, int mode) {
 
 	DICT_ATTR *attr;
 	VALUE_PAIR *pair, *check;
+	char *ptr;
+	char buf[128];
 
 	if ((attr = dict_attrbyname(row[2])) == (DICT_ATTR *) NULL) {
 		radlog(L_ERR | L_CONS, "rlm_sql: unknown attribute %s", row[2]);
@@ -261,10 +263,16 @@ int sql_userparse(VALUE_PAIR ** first_pair, SQL_ROW row, int mode) {
 			mode == PW_VP_GROUPDATA)
 		return 0;
 
-	pair = pairmake(row[2], row[3], T_OP_CMP_EQ);
-	pairadd(first_pair, pair);
+	if (row[4] != NULL && strlen(row[4]) > 0) {
+		ptr = row[4];
+		if ((mode = gettoken(&ptr, buf, sizeof(buf))) <= T_EOL)
+			mode = T_OP_CMP_EQ;
+	} else {
+		mode = T_OP_CMP_EQ;
+	}
 
-	vp_printlist(stderr, *first_pair);
+	pair = pairmake(row[2], row[3], mode);
+	pairadd(first_pair, pair);
 
 	return 0;
 }
