@@ -3,7 +3,7 @@
  *		accounting:     Functions to write radwtmp file.
  *		Also contains handler for "Group".
  *
- * Version:	$Id: rlm_unix.c,v 1.36 2001/10/01 18:22:52 hartwick Exp $
+ * Version:	$Id: rlm_unix.c,v 1.42 2001/11/09 00:53:11 kkalev Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
-static const char rcsid[] = "$Id: rlm_unix.c,v 1.36 2001/10/01 18:22:52 hartwick Exp $";
+static const char rcsid[] = "$Id: rlm_unix.c,v 1.42 2001/11/09 00:53:11 kkalev Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -32,6 +32,8 @@ static const char rcsid[] = "$Id: rlm_unix.c,v 1.36 2001/10/01 18:22:52 hartwick
 #include	<string.h>
 #include	<grp.h>
 #include	<pwd.h>
+#include	<sys/types.h>
+#include	<sys/stat.h>
 
 #include "config.h"
 
@@ -71,6 +73,9 @@ struct unix_instance {
 	const char *radwtmp;
 	int usegroup;
 	struct pwcache *cache;
+	time_t cache_reload;
+	time_t next_reload;
+	time_t last_reload;
 };
 
 static CONF_PARSER module_config[] = {
@@ -89,6 +94,8 @@ static CONF_PARSER module_config[] = {
 	  offsetof(struct unix_instance,radwtmp), NULL,   "${logdir}/radwtmp" },
 	{ "usegroup", PW_TYPE_BOOLEAN,
 	  offsetof(struct unix_instance,usegroup), NULL,     "no" },
+	{ "cache_reload", PW_TYPE_INTEGER,
+	  offsetof(struct unix_instance,cache_reload), NULL, "600" },
 	
 	{ NULL, -1, 0, NULL, NULL }		/* end the list */
 };
@@ -107,7 +114,31 @@ static struct unix_instance *group_inst;
  * file) or not ("Group=" was bound to the first instance of rlm_unix */
 static int group_inst_explicit;
 
-struct passwd *fgetpwnam(const char *fname, const char *name) {
+#if HAVE_GETSPNAM
+#if defined(M_UNIX)
+	typedef struct passwd shadow_pwd_t;
+	static inline const char *get_shadow_name(shadow_pwd_t *spwd) {
+		if (spwd == NULL) return NULL;
+		return (spwd->pw_name);
+	}
+	static inline const char *get_shadow_encrypted_pwd(shadow_pwd_t *spwd) {
+		if (spwd == NULL) return NULL;
+		return (spwd->pw_passwd);
+	}
+#else /* M_UNIX */
+	typedef struct spwd shadow_pwd_t;
+	static inline const char *get_shadow_name(shadow_pwd_t *spwd) {
+		if (spwd == NULL) return NULL;
+		return (spwd->sp_namp);
+	}
+	static inline const char *get_shadow_encrypted_pwd(shadow_pwd_t *spwd) {
+		if (spwd == NULL) return NULL;
+		return (spwd->sp_pwdp);
+	}
+#endif	/* M_UNIX */
+#endif	/* HAVE_GETSPNAM */
+
+static struct passwd *fgetpwnam(const char *fname, const char *name) {
 	FILE		*file = fopen(fname, "ro");
 	struct passwd	*pwd = NULL;
 
@@ -118,15 +149,18 @@ struct passwd *fgetpwnam(const char *fname, const char *name) {
 			fclose(file);
 			return NULL;
 		}
-	} while(strcmp(name, pwd->pw_name) != 0);
+	} while (strcmp(name, pwd->pw_name) != 0);
+
 	fclose(file);
 	return pwd;
 }
 
-struct passwd *fgetgrnam(const char *fname, const char *name) {
+static struct group *fgetgrnam(const char *fname, const char *name) {
 	FILE		*file = fopen(fname, "ro");
 	struct group	*grp = NULL;
+
 	if(file == NULL) return NULL;
+
 	do {
 		grp = fgetgrent(file);
 		if(grp == NULL) {
@@ -138,10 +172,30 @@ struct passwd *fgetgrnam(const char *fname, const char *name) {
 	return grp;
 }
 
+#if HAVE_GETSPNAM
+
+static shadow_pwd_t *fgetspnam(const char *fname, const char *name) {
+	FILE		*file = fopen(fname, "ro");
+	shadow_pwd_t	*spwd = NULL;
+
+	if(file == NULL) return NULL;
+	do {
+		spwd = fgetspent(file);
+		if(spwd == NULL) {
+			fclose(file);
+			return NULL;
+		}
+	} while(strcmp(name, get_shadow_name(spwd)) != 0);
+	fclose(file);
+	return spwd;
+}
+
+#endif
+
 /*
  *	The Group = handler.
  */
-static int groupcmp(void *instance, VALUE_PAIR *request, VALUE_PAIR *check,
+static int groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check,
 	VALUE_PAIR *check_pairs, VALUE_PAIR **reply_pairs)
 {
 	struct passwd	*pwd;
@@ -159,8 +213,6 @@ static int groupcmp(void *instance, VALUE_PAIR *request, VALUE_PAIR *check,
 		return 1;
 	}
 
-	radlog(L_ERR, "group = %s", group_inst->group_file);
-
 	username = (char *)request->strvalue;
 
 	if (group_inst->cache_passwd &&
@@ -232,6 +284,11 @@ static int unix_instantiate(CONF_SECTION *conf, void **instance)
 			free(inst);
 			return -1;
 		}
+
+		if (inst->cache_reload) {
+			inst->last_reload = 0;
+			inst->next_reload = time(NULL) + inst->cache_reload;
+		}
 	} else {
 		inst->cache = NULL;
 	}
@@ -296,11 +353,7 @@ static int unix_authenticate(void *instance, REQUEST *request)
 	char		*encrypted_pass;
 	int		ret;
 #if HAVE_GETSPNAM
-#if defined(M_UNIX)
-	struct passwd	*spwd;
-#else
-	struct spwd	*spwd;
-#endif
+	shadow_pwd_t	*spwd = NULL;
 #endif
 #ifdef OSFC2
 	struct pr_passwd *pr_pw;
@@ -314,6 +367,56 @@ static int unix_authenticate(void *instance, REQUEST *request)
 	char		*shell;
 #endif
 
+	/* See if we should refresh the cache */
+	if (inst->cache && inst->cache_reload
+	 && (inst->next_reload < request->timestamp)) {
+		/* Time to refresh, maybe ? */
+		int must_reload = 0;
+		struct stat statbuf;
+
+		DEBUG2("rlm_users : Time to refresh cache.");
+		/* Check if any of the files has changed */
+		if (inst->passwd_file
+		 && (stat(inst->passwd_file, &statbuf) != -1)
+		 && (statbuf.st_mtime > inst->last_reload)) {
+			must_reload++;
+		}
+
+		if (inst->shadow_file
+		 && (stat(inst->shadow_file, &statbuf) != -1)
+		 && (statbuf.st_mtime > inst->last_reload)) {
+			must_reload++;
+		}
+
+		if (inst->group_file
+		 && (stat(inst->group_file, &statbuf) != -1)
+		 && (statbuf.st_mtime > inst->last_reload)) {
+			must_reload++;
+		}
+
+		if (must_reload) {
+			/* Build a new cache to replace old one */
+			struct pwcache *oldcache;
+			struct pwcache *newcache = unix_buildpwcache(
+							inst->passwd_file,
+							inst->shadow_file,
+							inst->group_file);
+
+			if (newcache) {
+				oldcache = inst->cache;
+				inst->cache = newcache;
+				unix_freepwcache(oldcache);
+
+				inst->last_reload = time(NULL);
+			}
+		} else {
+			DEBUG2("rlm_users : Files were unchanged. Not reloading.");
+		}
+
+		/* Schedule next refresh */
+		inst->next_reload = time(NULL) + inst->cache_reload;
+	}
+
 	/*
 	 *	We can only authenticate user requests which HAVE
 	 *	a User-Name attribute.
@@ -373,8 +476,15 @@ static int unix_authenticate(void *instance, REQUEST *request)
 #else /* OSFC2 */
 	/*
 	 *	Get encrypted password from password file
+	 *
+	 *	If a password file was explicitly specified, use it,
+	 *	otherwise, use the system routines to read the
+	 *	system password file
 	 */
-	if ((pwd = getpwnam(name)) == NULL) {
+	if (inst->passwd_file != NULL) {
+		if ((pwd = fgetpwnam(inst->passwd_file, name)) == NULL)
+			return RLM_MODULE_NOTFOUND;
+	} else if ((pwd = getpwnam(name)) == NULL) {
 		return RLM_MODULE_NOTFOUND;
 	}
 	encrypted_pass = pwd->pw_passwd;
@@ -383,13 +493,25 @@ static int unix_authenticate(void *instance, REQUEST *request)
 #if HAVE_GETSPNAM
 	/*
 	 *      See if there is a shadow password.
+	 *
+	 *	If a shadow file was explicitly specified, use it,
+	 *	otherwise, use the system routines to read the
+	 *	system shadow file.
+	 *
+	 *	Also, if we explicitly specify the password file,
+	 *	only query the _system_ shadow file if the encrypted
+	 *	password from the passwd file is < 10 characters (i.e.
+	 *	a valid password would never crypt() to it).  This will
+	 *	prevents users from using NULL password fields as things
+	 *	stand right now.
 	 */
-	if ((spwd = getspnam(name)) != NULL)
-#if defined(M_UNIX)
-		encrypted_pass = spwd->pw_passwd;
-#else
-		encrypted_pass = spwd->sp_pwdp;
-#endif	/* M_UNIX */
+	if (inst->shadow_file != NULL) {
+		if ((spwd = fgetspnam(inst->shadow_file, name)) == NULL)
+			encrypted_pass = get_shadow_encrypted_pwd(spwd);
+	} else if ((encrypted_pass == NULL) || (strlen(encrypted_pass) < 10)) {
+		if ((spwd = getspnam(name)) != NULL)
+			encrypted_pass = get_shadow_encrypted_pwd(spwd);
+	}
 #endif	/* HAVE_GETSPNAM */
 
 #ifdef DENY_SHELL
@@ -415,8 +537,11 @@ static int unix_authenticate(void *instance, REQUEST *request)
 		}
 	}
 	endusershell();
-	if (shell == NULL)
+	if (shell == NULL) {
+		radlog(L_AUTH, "rlm_unix: [%s]: invalid shell [%s]",
+			name, pwd->pw_shell);
 		return RLM_MODULE_REJECT;
+	}
 #endif
 
 #if defined(HAVE_GETSPNAM) && !defined(M_UNIX)
@@ -461,9 +586,10 @@ static int unix_authenticate(void *instance, REQUEST *request)
 	 *	Check encrypted password.
 	 */
 	encpw = crypt(passwd, encrypted_pass);
-	if (strcmp(encpw, encrypted_pass))
+	if (strcmp(encpw, encrypted_pass)) {
+		radlog(L_AUTH, "rlm_unix: [%s]: invalid password", name);
 		return RLM_MODULE_REJECT;
-
+	}
 	return RLM_MODULE_OK;
 #endif /* OSFSIA */
 #undef inst
